! cyoa.h
! A Choose Your Own Adventure library for inform
! by Paolo Lucchesi, 2004
!
! This file is free software, it can be freely used, copied, distributed and
! modified. Every source distribution of this file, modified or not, should
! acknoledge the original author, and should be as free as the original.
! There's no such condition for compiled games using this library.
! This file is distributed AS IS, without warranty of any kind. The author 
! isn't liable for any direct or indirect damage due to the proper or improper 
! use of this software. 
! ------------------------------------------
! The inform standard library is not used. It's requested just the inform
! compiler v. 6.30 or later.
! The game file should begin with the following line
!
! #include "cyoa";
!
! There are two kind of objects (and therefore two classes) in the game: Nodes 
! and Options: nodes are game situations and have the following format
!
! Node <nodename> "Node's short description"
!    with text <longdescription>;
!
! where <longdescription> can be a string or a routine.
! 
! Options are the possible choices in a given node; they should be children
! of this node and can have one of the two following formats
!
! Option [optionname] "Option's description"
!    with goto <nextnode>;
!
! Option [optionname] "Option's description"
!    with action <actiontodo>;
!
! where <nextnode> is the name of the node to be reached if the option is
! choosen, and <actiontodo> is a string or routine to be printed or
! executed it the option is choosen. [optionname] is not mandatory.
!
! The moveto(<newnode>) routine selects <newnode> as the new current
! situation. The game starts with the Init routine (to be defined by the
! author), that should include at least the statement
!
! moveto(<firstnode>);
!
! The library can handle save, restore and quit by the S, R and Q keys.
! The library doesn't work under glulx.
! See cyoatest.inf for an example.
!
! ------------------------------------------

Attribute general; !--- An all purpose attribute usable by the author

Global stat=0; !--- Current node

Global nopt=0; !--- Number of options
Array opt --> 9; !--- Options' array

!--- Nodes Class
Class Node
  with text "",
  ndo [x i k;
      !--- Print name and text of the node
      style reverse;
      print (name) self;
      style roman;
      print "^";
      self.text();
      !--- Read all Options children of the node and stores them in opt
      nopt=0;
      objectloop (x in self) {
          if (x ofclass Option && nopt<9) {
              opt-->nopt=x;
              nopt++;
          }
      }
      !--- No option: quit
      if (nopt<=0) { stat=0; rfalse; }
      !--- Print the option list
      print "^You can:^";
      for (i=0: i<nopt: i++) print i+1, ". ", (name) opt-->i, "^";
      !--- Wait for the player to press an option number, value in k
      print ">>> ";
      while(true) {
          k=getchar();
          if (k == 83 or 82 or 81) break;
          if (k == 115 or 114 or 113) { k=k-32; break; }
          if (k > 48 && k <= (48+nopt)) { k=k-48; break; }
      }
      if (k == 83) { save rsdest; rtrue; }
      if (k == 82) { restore rsdest; rtrue; }
      if (k == 81) quit;
      print k, "^^";
      !--- run the odo properties of the choosen option
      k=k-1;
      x=opt-->k;
      x.odo();
.rsdest;
      rtrue;
  ];

Class Option
  with odo [;
      !--- run action properties, if it exists
      if (self provides action) { self.action(); ""; }
      !--- else the value of goto properties is the new node
      if (self provides goto) stat=self.goto();
  ];

!--- Warning crusher
Object xdummy
   has general;

!--- Main routine, everything starts here
[ main;
    Init();
    !--- While stat is a node, run its ndo properties
    while (stat ofclass Node) {
        stat.ndo();
    }
];

!--- moveto(x) move player to node x
[ moveto x;
    stat=x;
];

!--- getchar() wait for a keypress, return value
[ getchar k; @read_char 1 -> k; return k; ];
