!-----------------------------------------------------------------------------
! Magic and Fight System --- Offensive spells
! For INFORM compiler and Z-Machine text based Adventure games
!-----------------------------------------------------------------------------
!  Author: Giancarlo Niccolai
!  Version: 1.3
!  Last Revised: 2002-11-24
!  Contact author at:
!  Feel free to use and distribute this file, but please report me if
!  you intend to change it (in better), so we can upgrade together
!
!  Released under GPL 2.0, or any later version at your will
!
!-----------------------------------------------------------------------------
!
! Every object or character which could be damaged must
! have a "damage" routine, with parameters:
!  - the amount of damage
!  - the source of the the damage (which is the spell,
!    normally).
! So the target can be resistant to certain damage
! sources. The target may also test if the source is from a certain
! class by refering to the "sender" operator; in fact, the sender of the
! damage message is also the source. A Fireball spell and a Flame Thrower
! will cause both fire type damage, but hopefully they will be of two
! different classes: the operator 'sender ofclass' will indicate which fire
! is generated from the spell, and which from a war weapon.
!------------------------------------------------------------------------------

Ifndef AttackSpell;
	include "MFS_Magic.h";
Endif;

!----------------------------------------------------------------
! Causes a fire bolt to be hurled to a target
!----------------------------------------------------------------
AttackSpell ferdem_spell "ferdem"
    with
	name "ferdem",
	damage_type DMG_FIRE,
    purpose "getta un dardo di fuoco su di un bersaglio",
    description [;
        print "Questo incantesimo provocher un discreto danno ad un avversario
		  o ad un oggetto che possa essere incendiato.^";
    ],

    casting_time 2,
    cost 2,
    power 1,

	min_damage 5,
	max_damage 12,

	effect [stage target;
		print "Una piccola sfera di fuoco appare di fronte a te, danzando nell'aria per
		qualche istante. ";

		switch(stage) {
		    3: "E quando la magia decide che non ci sono bersagli nelle vicinanze, la sfera
			 sparisce in una nuvoletta di fumo.";
			2: print "Improvvisamente, la sfera piomba ",
            (artsu) target,"!^";
				"Il ferdem colpisce senza fallo: ",
				(youthe) target, " ",(isorare) target,
				" circondat", (gan) target," dalle fiamme!";
			1: print "Improvvisamente, la sfera piomba colpisce ",
					(youthe) target,",";
				" ma ", (youthe) target,
				(isorare) target,  " immune all'incantesimo.";
			0: "... ma il fuoco si dissipa in una nebbiolina impalpabile, senza danneggiare alcunch.^";
		}
	];

!----------------------------------------------------------------
! Causes a lightning to be hurled to a target
!----------------------------------------------------------------
AttackSpell spindem_spell
    with
	name "spindem",
	damage_type DMG_ELECTRICITY,
    purpose "scaglia un terribile fulmine verso un bersaglio, causando un danno rilevante",
    description [;
        print "Questo incantesimo causer un notevole danno all'oggetto pi vicino, se lanciato
		  senza un bersaglio specifico, oppure ad un preciso oggetto. Ricorda che il fulmine 
		  composto da elettricit.^";
    ],

    casting_time 3,
    cost 4,
    power 4,

	min_damage 15,
	max_damage 25,

	effect [ stage target;
		if (target > 0)
	 	print "Odi il rombo di un tuono, ed il luogo in cui ti trovi  abbagliato dal
		chiarore del fulmine che si abbate ",(artsu) target,".^";

		switch(stage) {
    		3: "Un tuono si ode, ma, per il fulmine, non c'@`e nulla da colpire.
				Cos@`i la magia ignora il tuo comando.
				Null'altro accade.";
			2: if ( target ~= player)
               "Capisci che ", (the) target, " ", (isorare) target,
               " stat", (gan) target, " colpit", (gan) target, ".";
            else
               "Provi un dolore lancinante e i tuoi muscoli si contraggono
                  in uno spasmo di dolore";

			1: "Ma niente sembra essere accaduto ",
				(arta) target,": ",(pron) target, " deve essere immune ai fulmini.^";
			0: "... ma il fulmine non ha causato danni.";
		}
	];

!----------------------------------------------------------------
! Causes an explosion to invest the target
!----------------------------------------------------------------
AttackSpell bumdem_spell "bumdem"
    with
	name "bumdem",
	damage_type DMG_HITWEAPON,
    purpose "provoca un'esplosione, causando un enorme danno",
    description [;
        print "Se lanciato senza bersaglio, questo incantesimo causer@`a un enorme danno
		  al nemico pi@`u vicino, altrimenti colpir@`a il bersaglio prefissato.^";
    ],

    casting_time 6,
    cost 6,
    power 8,

	min_damage 25,
	max_damage 50,

	effect [stage target;
		if (target ~= nothing )
			print "Un fragore terribile riempie l'aria scossa
			dal lampo del potere che cade ",(artsu) target,".^";

		switch(stage) {
			3: "La terra trema, ma non c'@`e nulla che debba essere fatto esplodere.
			Cos@`i l'incantesimo di bumdem decide di non fare nulla.^";
			2: "Ti accorgi che ", (youthe)target, " ",
				(isorare) target," stato colpito.";
			1: print "Ma pare che ", (youthe) target, " sia immune all'incantesimo.";
			0: "... ma l'esplosione non causa alcun danno.";
		}
	];


!----------------------------------------------------------------
! Hits all the hostile or neutral targets with a gust of fire
!----------------------------------------------------------------
AttackSpell girdum_spell "girdum"
    with
	name "girdum",
	damage_type DMG_FIRE,
    purpose "sfera di fuoco che danneggia tutto ci@`o che si trova intorno",
    description [;
        print "Questo incantesimo infligge un discreto quantitativo di danni
		  a tutti gli oggetti ed i nemici che si trovano nei paraggi. Non richiede
		  un bersaglio specifico^";
    ],

    casting_time 4,
    cost 8,
    power 6,

	min_damage 8,
	max_damage 16,
	target_type MFS_MULTIPLETARGET,

	effect [stage target;
		if (target ~= nothing ) {
			print "Investit", (gan) target, " da un'onda di fuoco, ";
			if ( target == player) print "assaggi";
			else print (the) target, " assaggia";
			print " il teribile calore.^";
		}

		switch(stage) {
	 		3: "Una graziosa sfera di fuoco brucia il contenuto di questa
				zona, ma siccome non c'@`e nulla da bruciare, non accade nulla.";
			2: "Il fuoco investe ",(youthe) target, " in pieno!";
			1: print "Ma ", (youthe) target, " ", (isorare) target, " immune al fuoco,
            e non sembra subire alcun danno!";
			0: "... ma il fuoco non dannegger@`a ",
				(youthe) target,".";
		}
	];

! end of attack spells.h
