!-----------------------------------------------------------------------------
! Magic And Fight System --- Writing spells
! For INFORM compiler and Z-Machine text based Adventure games
!-----------------------------------------------------------------------------
!  Author: Giancarlo Niccolai
!  Version: 1.3
!  Last Revised: 2002-11-24
!  Contact author at: giancarlo@niccolai.ws
!  Feel free to use and distribute this file, but please report me if
!  you intend to change it (in better), so we can upgrade together
!
!  Released under GPL 2.0, or any later version at your will
!
!-------------------------------------------------------------------------------
! It includes three classes: the MagicalWiting Device,
! used to scribe scrolls, the Scroll, used to cast and learn
! spells and the MagicBook, used only to learn spells.
! Scrolls and Books have a base class: the SpellWriteable class,
! which sould be the base class for every object that is
! capable of putting spells in memory (learning).
!-------------------------------------------------------------------------------

Ifndef SpellWriteable;

Ifndef can_cast;
	include "MFS_Magic.h";
Endif;

Attribute legible;

!--------------------------------------------------------------------------
! Magical writing device class declaration
! If the player wants to write a spell into a magical device, he must own
! an object derived from this class.
!
Class MagicalWritingDevice;

!--------------------------------------------------------------------------
! Writable objects that can carries spells
! Spells either come from an object of this kind or are casted from this
!
class SpellWriteable
with
	content 0 0,  ! to be filled with spells or space!

	! 'read' in intercepted, while 'examine' is not,
	! and prints the "description" (if provided)

	before [;
		Examine:
				if (verb_word == 'leggi' || verb_word == 'leggo')
					if (noun notin player) {
						print "(Prima prendendo ",(the) self,").^";
						<Take self>;
						<<Consult self>>;
					}
					else {
						<<Consult self>>;
					}
				else rfalse;

		Consult:
			print "Questo @`e il contenuto ",(artdi) self,": ^^";
			self.describe_contents();
			rtrue; ! ci ho gia' pensato io.
	],


	!here you must add a 'content' propriety, which is
	! an array.
	has_spell[ spell i;
		for (i=0: i < self.#content/2: i++) {
				if (self.&content-->i == spell) rtrue;
		}
		rfalse;
	],

	is_empty[ i;
		for (i = 0: i<self.#content/2 : i++)
				if (self.&content-->i ~= 0) return false;
		return true;
	],

	add_spell [spell i;
		for (i = 0: i<self.#content/2 : i++)
				if (self.&content-->i == 0) {
					self.&content-->i = spell;
					rtrue;
				}
		rfalse;
	],

	describe_contents[i obj;
		for (i = 0: i < self.#content/2 : i++) {
			obj = self.&content-->i;
				if ( obj ~= 0) {
					if (obj provides specification) {
					obj.specification();
					}
				}
		}
	],

	remove_spell [spell i;
		for (i = 0: i < self.#content/2:i++)
			if (self.&content-->i == spell) {
				self.&content-->i = 0;
				rtrue;
			}
		rfalse;
	],

	scribe_spell [sp;
		if (self.add_spell(sp))
			"^Come tracci i simboli mistici nella pergamena, vedi l'inchiostro brillare di una
			debole luce pulsante. Quindi, l'ultima parola ", (artdi) sp," @`e scritta ",
			(artsu) self, ".";
		else {
			print "Questo oggetto magico  stato scritto interamente, e non @`e rimasto
			spazio libero.^";
			rfalse;
		}
	],

	learn [spell;
		! normally, nothing to do...
		if (self.has_spell(spell)) rtrue;
		rfalse;
	];


!----------------------------------------------------------
! Scrolls are spell sources on which spells can be written
!

Class Scroll class SpellWriteable
	with
	writable 1,				! Still can be written
	prepared 0,				! Is ready to use ?
	!give a default content array
	content 0 0  0 0  0 0  0 0  0 0,

	name "pergamena", article "una",
	description "Una pergamena completamente bianca",

	! spell provider routines:
	can_cast[sp;
		if (self.has_spell(sp) && self.prepared == 1)
			return MFS_SCROLL_PRIORITY;
		return 0;
	],
	cast[sp tr rm caster;
		return self.unfold_one(sp, tr, rm, caster);
	],

	! spell learning source routine
	can_learn[sp ;
		if (self.has_spell(sp))
			return MFS_SCROLL_PRIORITY;
		return 0;
	],

	learn [spell;
		self.remove_spell(spell);
		self.writable = 0;
		if ( actor == player )
			print "Quando ",(the) spell," entra nella tua mente, 
				le parole ", (artsu) self , " scompaiono.^";

		if (self.is_empty()) {
			remove (self);
			if ( actor == player )
				print "Nel momento in cui l'ultimo incantesimo che vi era scritto viene usato, ",
				(the) self, " svanisce senza lasciare traccia.^";
		}
	],

	unfold [ ro cas i;
		if (ro == 0) ro = location;
		if (cas == 0) cas = player;
		if (self.is_empty()) {
			if (caster == player && TestScope( self, player ) )
				"La pergamena vuota non pu essere dispiegata!";
			return;
		}

		! unfolding a scroll causes the immediate release of all the spells in it
		if (cas == player )
			if ( TestScope(self, player) )
				print (The) self, " @`e srotolata, e tutti gli incantesimi che vi si trovano scritti
				vengono lanciati contemporaneamente!^^";
			else
				print "I tuoi sensi ti avvisano che ", (the) self,
               " @`e stata srotolata.^";
		for (i = 0: i<self.#content/2 : i++)
			if (self.&content-->i ~= 0) {
				if (TestScope(self)) {
					style bold;
					print (The) self.&content-->i;
					style roman;
					print ":^";
				}
				if ( (self.&content-->i) provides fire_to_all) {
					(self.&content-->i).caster = cas;
					(self.&content-->i).room = ro;
					<FireSpell (self.&content-->i)>;
				}
				else
					if (TestScope(self))
						print "Questo incantesimo non pu@`o essere usato senza bersaglio.^";
			}
			else
				if ( (self.&content-->i) provides fire_to_all) {
					(self.&content-->i).caster = cas;
					(self.&content-->i).room = ro;
					<FireSpell (self.&content-->i)>;
				}

		if ( caster == player && TestScope(self) )
		print "^La pergamena ha rilasciato tutto il proprio potere, ed ora la sua carta magica
		si sbriciola!^";
		remove (self);
	],

	unfold_one[ spell tr ro cas ;
		if (ro == 0) ro = location;
		if (cas == 0) cas = player;
		if (cas == player)
			print "Dispieghi ", (the) self, ", cercando ", (the) spell,".^^";
		else if (TestScope(caster)) {
			print (The) caster, " spiega ", (a) self,
				", cercando ", (SpellName) spell, "!^^";
		}

		if (tr == 0 && spell provides fire_to_all) {
			spell.caster = cas;
 			spell.room = ro;
			<FireSpell spell>;
		}
		else if (tr ~= 0 && spell provides fire)
			<FireSpell spell tr>;
		else if (player == cas)
			"L'incantesimo che hai richiamato necessita un bersaglio; non funzioner@`a cos@`i.";

		self.writable = 0;
		self.remove_spell(spell);
		if (self.is_empty()) {
				remove (self);
			if (player == caster)
				print "^La pergamena ha rilasciato tutto il proprio potere, ed ora la sua carta magica
					si sbriciola!^";
		}
	],

	scribe_spell[sp obj;
		objectloop (obj in player)
			if (obj ofclass MagicalWritingDevice) {
				if (self.writable == 0) {
					print "Questa pergamena @`e stata magicamente sigillata, e non vi si pu@`o pi@`u
					scrivere.^";
					rfalse;
				}
				self.SpellWriteable::scribe_spell(sp);
				rtrue;
			}

		print "Dai tempi dell'Accademia, sei in grado di ricordare che hai bisogno di
			un oggetto magico particolare per scrivere gli incantesimi sulle pergamene!^";
		rfalse;

	],

	prepare [ obj;
		if (self.prepared == 1)
			if ( actor == player ) "Beh, lo hai gi@`a fatto!";
			else rtrue;

		objectloop(obj in actor && obj ofclass Scroll)
			if (obj.prepared == 1)
				if ( actor == player )
					print (The) obj, " @`e stata precedentemente preparata,
					ed @`e possibile preparare una sola pergamena alla volta.^";
				else
					rtrue;
		self.prepared = 1;
		if ( actor == player )
			"Adesso ", (the) self, " @`e pronta per rilasciare il proprio potere!";
		rtrue;
	],

	repose [;
		if (self.prepared == 1) {
			self.prepared = 0;
			if (actor == player )
				"Riponi ", (the) self, ".";
		}
		if ( actor == player )
			"Ma ", (the) self, " non @`e stata preparata!";
		rtrue;
	],

	invent[;
		if (inventory_stage==2 && self.prepared == 1)
			print " (pronta per lanciare incantesimi)";
		rfalse;
	],

	! after a Drop action, a scroll can't be prepared
	after [;
		Take: Drop:
			self.prepared = 0;
			rfalse;
	]
	has female;


!--------------------------------------------------------------------------
! A specialized spell source
!
Class MagicBook class SpellWriteable
	with
	name "libro" "magico",

	description "@`E un elegante tomo, con decorazioni dorate e la copertina pesante.",

	! add space for 16 spells. Note that books found on
	! the way can be less capable.
	content 0 0 0 0 0 0   0 0 0 0 0 0   0 0 0 0 0 0
			0 0 0 0 0 0   0 0 0 0 0 0   0 0,

	can_learn[sp ;
		if (self.has_spell(sp))
			return MFS_BOOK_PRIORITY;
		return 0;
	],

	scribe_spell[ sp;
		if (self.has_spell(sp)) {
			print (The) sp, " @`e gi@`a stato scritto  ", (artsu) self,".^";
			rfalse;
		}
		self.SpellWriteable::scribe_spell(sp);
	];

Endif; ! ifndef SpellWriteable

! End of MFS_SpellWriteable.h
