!-----------------------------------------------------------------------------
! Magic and Fight System --- Standard spells collection
! For INFORM compiler and Z-Machine text based Adventure games
!-----------------------------------------------------------------------------
!  Author: Giancarlo Niccolai
!  Version: 1.3
!  Last Revised: 2002-11-24
!  Contact author at: giancarlo@niccolai.ws
!  Feel free to use and distribute this file, but please report me if
!  you intend to change it (in better), so we can upgrade together
!
!  Released under GPL 2.0, or any later version at your will
!
!-----------------------------------------------------------------------------
! This nice collection of spells can be freely used in your adventures.
! Some spells requires the MFS_MagicObject to be included; if not, they
! won't be available.
!------------------------------------------------------------------------------

Ifndef Spell;
   Include "MVS_Magic.h";
Endif;

!-----------------------------------------------------------------------
! Writes a spell from a scroll to a book. You can use this in alterative
! with the magical writer object
! The spell has for_free attribute, and can be used without any limits
! Useable by players only
!-----------------------------------------------------------------------

Spell gnusto_spell
   with
   name "gnusto",

   power 1,
   casting_time 0,
   cost 0,
   purpose "Scrive un incantesimo su un tomo",

   fire [target obj ob1 found;
      found = false;
      objectloop (obj ofclass MagicBook &&
         obj in actor) {
         found = true;
         break;
      }

      if (~~found)
         "L'incantesimo girovaga attorno alle tue mani, in cerca di un libro magico,
         e non trovandolo, scompare riluttante.";

      if (~~ (target ofclass Spell) )
         "Non puoi scrivere ", (the) noun, " su di un libro degli incantesimi!";

      if (obj.has_spell(target))
         "L'incantesimo @`e gi@`a stato scritto sul libro!";

      found = false;

      objectloop(ob1 in actor) {
         if (ob1 ofclass Scroll &&
         ob1.has_spell(target)) {
            found = true;
            break;
         }
      }
      if (~~found)
         "Devi avere un incantesimo su di una pergamena perch@`e questo possa essere scritto.";

      if (obj.add_spell(target)) {
         print "Il tuo libro degli incatesimi comincia a risplendere delicatamente. Lentamente,
         le parole ", (artdi) target, " vengono trascritte, e risplendono ancora pi@`u forte
         del libro stesso. Il chiarore del libro scompare, ma l'incantesimo rimane!^Comunque,
         le parole dell'incantesimo ", (artsu) scroll, " svaniscono come l'ultima di esse viene
         copiata";

         ob1.remove_spell(target);
         if (ob1.is_empty()) {
            remove ob1;
            print ", e siccome questo era l'unico incantesimo scritto sulla pergamena,
               essa diventa polvere";
         }
         print ".^";
      }
      else {
         "Il tuo libro degli incantesimi @`e pieno, e non pu@`u accettare nuovi incantesimi.";
      }
   ],
   has for_free;


!-----------------------------------------------------------------------
! Gives light to an object or a location
! Useable by players only
!-----------------------------------------------------------------------
Spell litdem_spell "Litdem"
   with name "litdem",
   purpose "illumina un luogo o un oggetto inanimato",

   fire [target;
      if (target has animate)
         "Una luce si libra per un po' ", (artsu)
            target, " ... ma proprio quando l'incantesimo svanisce,
            ti ricordi che il litdem non funziona sulle creature animate.";

      give target light;
      print (The) target ," inizia a brillare...^";
   ],

   fire_to_all [;
      if (self.room has light )
         "Vi era gi@`a luce in questo luogo...
         quindi, l'incantesimo non ha effetto.";

      if (self.room==thedark)
         give real_location light;
      else
         give self.room light;
      "Questo luogo @`e ora illuminato!";

   ];

!-----------------------------------------------------------------------
! Reveals hidden magical proprieties of a target object
! Useable by players only
!-----------------------------------------------------------------------
Spell shtratlim_spell
   with name "shtratlim",
   purpose "rivela propriet@`a magiche nascoste",
   casting_time 1,
   cost 1,
   power 2,

   description "Questo incantesimo rivela poteri magici nascosti posseduti dagli
      oggetti. Comunque, alcuni oggetti ~intelligenti~ possono tenere nascoste
      comunque le proprie capacit, e rivelarle solo se, in qualche modo, sono
      ~consenzienti~.",

   fire [target;
      if (~~ AffectTarget(target, SPELL_REVEAL))
         "Grazie al tuo potente incantesimo, scopri che ",
         (the) target ," non ha capacit@`a magiche nascoste...^
         ... O quantomeno, non ha propriet@`a che questo incantesimo possa rivelare.";
   ];

!-----------------------------------------------------------------------
! Recharges magical devices, sending a "SPELL_RECHARGE" message to
! the target.
! Useable by players only
!-----------------------------------------------------------------------
Spell zemar_spell
   with name "zemar",
   purpose "ricarica gli oggetti magici",
   casting_time 5,
   cost 4,
   power 3,

   description 
      "Se qualche oggetto magico ha un potere limitato, e pu@`o essere usato per
      un certo numero di volte, questo incantesimo fornir@`a all'oggetto un po' di
      energia. Ad ogni modo, alcuni oggetti sono inutilizzabili una volta usati,
      e non vi @`e modo di scoprirlo se non lanciando lo zemar su di essi.",

   fire [target;
      print "Le tue mani cominciano a brillare, e senti una grande forza attraversarti.
         quindi", (the) target, " @`e improvvisamente colpito da un raggio di luce
         promanato dalle tue mani.^";

      if (~~AffectTarget(target,SPELL_RECHARGE))
         "Quando tutto @`e finito, ti rendi conto che ",
            (the) target, " @`e inerte. Ti sembra ovvio (ora) che l'oggetto
            non pu@`o essere ricaricato.";
   ];


!-----------------------------------------------------------------------
! Utterly useless spell... just for debugging purpose or for printing
! out your copyright.
! Useable by players only
!-----------------------------------------------------------------------
Spell filfre_spell "filfre"
   with name "filfre",
   purpose "crea graziosi fuochi d'artifico",
   casting_time 0,
   cost 0,
   power 0,
   copyrights "	Un file senza copyright",

   fire_to_all [;
      print "Quando termini di pronunciare le parole magiche, un simpatico fuoco d'artificio
      compare nell'aria; la cenere luminescente compone le parole:^^";
      style bold;
      print (string) self.copyrights;
      style roman;
      "^^Quando l'aria torna limpida, tu rimani un poco dubbioso. Quindi squoti la testa
      sorridendo e continui il tuo viaggio.^";
   ];


!-----------------------------------------------------------------------
! Removes a curse upon things by sending a "UNCURSE" affect message
! Useable only by player
!-----------------------------------------------------------------------
Spell nesinit_spell "nesinit"
   with
   name "nesinit",
   power 2,
   purpose "rimuove le maledizioni sugli oggetti",
   description 
      "Alcuni oggetti magici possono subire l'imposizione di maledizioni di vario genere,
      e per vari motivi. Questo incantesimo cancella gran parte delle maldizioni che 
      possono essere state intessute su ogni tipo di oggetto.^",

   fire[target;
      print (The) target, " viene colpito da una luce azzurrina...^";
      if (~~AffectTarget(target,SPELL_UNCURSE))
      "...ma dato che nessuna maledizione affligge l'oggetto, nulla accade.";
   ];

!-----------------------------------------------------------------------
! Teleport the player to a location that has been previously memorized
! using the banzor spell
! Useable only by player
!-----------------------------------------------------------------------
Spell banzor_spell "banzor"
with
   name "banzor",

   power 4,
   casting_time 4,
   cost 3,
   memorized 0,

   purpose "memorizza un luogo",

   description [;
      print "Dopo aver lanciato il banzor, puoi usare alcuni incantesimi che
         agiranno sul luogo memorizzato, anche se ti trovi molto lontano
         o persino se sei su un altro piano dimensionale. Puoi memorizzare solo
         un luogo alla volta. Per usare il banzor, basta lanciarlo
         senza alcun bersaglio";
      if ( self hasnt general )
         print "; il luogo in cui ti trovi verr@`a cos@`i memorizzato.^";
      else
         print ", memorizzando in questo modo il luogo in cui ti trovi,
            oppure puoi usare come bersaglio un luogo che sei in grado
            di vedere. ";

      if (self.memorized ~= 0) {
         print "In precendeza, hai banzorizzato ";
         style bold;
         print (the) self.memorized;
         style roman;
         ".";
      }

   ],

   before [;
      Cast:
         if (self hasnt general && second ~= nothing )
            "Il banzor pu@`o essere usato solo senza bersagli.";
         else if ( second ~= nothing && ~~ AffectTarget( second, 0 ) )
            "Il banzor pu@`o essere lanciato senza un bersaglio,
            oppure avendo come bersaglio un
            determinato luogo che sei in grado di vedere.";
         rfalse;
   ],

   fire [ loc flag ;
      if (banzor_spell.memorized == 0) flag ==1;
      banzor_spell.memorized = loc;
      print "Adesso ogni singolo dettaglio ", (artdi) banzor_spell.memorized, " @`e fissato
         nella tua mente";

      if (flag == 1)
         "; nessuna traccia rimane del precedente luogo banzorizzato, nemmeno il suo nome";

      ".";
   ],

   fire_to_all [flag ;
      if (self.memorized == 0) flag ==1;

      if ( self.room == thedark)
         self.memorized = real_location;
      else
         self.memorized = self.room;
      print "Adesso ogni singolo dettaglio ", (artdi) banzor_spell.memorized,
         " @`e fissato nella tua mente.";

      if (flag == 1)
         "; nessuna traccia rimane del precedente luogo banzorizzato, nemmeno il suo nome.";
      else
         ".";
   ];

!-----------------------------------------------------------------------
! Teleport the player to a location that has been previously memorized
! using the banzor spell
! Useable only by player
!-----------------------------------------------------------------------
Spell vagnit_spell "vagnit"
with
   name "vagnit",
   power 6,
   casting_time 5,
   cost 4,

   purpose "teletrasporta un oggetto in un luogo memorizzato",

   description "Questo incantesimo teletrasporta un oggetto o una creatura in
   un luogo precedentemente memorizzato grazie all'incantesimo di banzor. Per
   teletrasportare un oggetto @`e necessario prenderlo in mano, quindi @`e impossibile
   teletrasportare oggetti grandi o fissati al suolo.",

   fire [target;
      if (target ~= player)
         print "Una scarica di potere emerge dalle tue mani puntate ", (artsu) target, "
         e ", (itorthem) target, "inizia a vibrare e tremare.^";
      else
         print "Ti senti un poco confuso...^";

      if (target has static)
         " ...ma ...nulla accade!^", (The) target, " l'oggetto @`e fissato in qualche modo,
         e non si muover@`a.^";

      if (banzor_spell.memorized == 0)
         " ...ma ...nulla accade!^Non hai memorizzato alcun luogo con l'incantesimo di banzor.^" ;

      if (target ~= player) {
         move target to banzor_spell.memorized;
         ", e subitamente scompare con un suono simile ad un ~pop~!^";
      }
      else
         PlayerTo(banzor_spell.memorized);
   ];

!player only
Spell vedonit_spell "vedonit"
with
   name "vedonit",

   power 2,
   cost 1,
   casting_time 0,

   purpose "mostra un luogo memorizzato",
   description "Una visione del luogo memorizzato grazie all'incantesimo di banzor
   apparir@`a brevemente nell'aria.^",

   fire_to_all[;
      if (banzor_spell.memorized == 0)
         "Non hai memorizzato alcun luogo con l'incantesimo di banzor.^";

      print "...Una visione appare di fronte a te...^";
      print "Vedi ";
      style bold;
      print (the) banzor_spell.memorized;
      style roman;
      print ".^";
      if (banzor_spell.memorized provides description)
         PrintOrRun (banzor_spell, memorized.description);

      "^...La visione scompare...^";
   ];

!-----------------------------------------------------------------------
! A Spell causing the opening of a portal
! --- Requires the MFS_MagicObjects.h to be included
! Useable only by player
!-----------------------------------------------------------------------

Ifdef MagicalPortal;
Spell dordum_spell "dordum"
with
   name "dordum",
   power 5,
   casting_time 4,
   cost 4,
   going 0,
   portal,

   purpose "Apre un portale attraverso cui pu@`o essere raggiunta una locazione memorizzata",

   description "Questo incantesimo @`e in grado di creare un portale magico collegato
   ad una locazione memorizzata grazie all'incantesimo di banzor. Il portale durer@`a
   alcuni minuti, e poi svanir@`a.",

   fire_to_all [;
      if (self.going ~=0)
         "Non puoi creare un portale mentre un altro @`e ancora aperto.^";

      if (banzor_spell.memorized == 0)
         "Senza aver prima memorizzato alcun luogo con l'incantesimo di banzor, nulla
         pu@`o accadere.^";


      self.portal = MagicalPortal.create();
      if (self.portal == 0)
         "ERRORE DI PROGRAMMAZIONE: troppi portali in gioco.^";

      ! twelve turns before the portal is going
      self.going = 12;
      if (location == thedark) move self.portal to real_location;
      else move self.portal to location;
      self.portal.other_place = banzor_spell.memorized;
      StartDaemon(self);
      "Un magnifico portale magico appare!^";
   ],

   daemon [flag;
      self.going--;
      if (TestScope(self.portal)) flag = 1;
      if (self.going == 0) {
         remove self.portal;
         MagicalPortal.destroy(self.portal);
         StopDaemon(self);
      }
      if (flag == 0) return;
      switch (self.going) {
         2: "Il portale magico sembra tremolare.";
         1: "Il portale magico sta sbiadendo; non durer@`a ancora a lungo.";
         0: "Il portale magico @`e sparito.";
      }
   ];
Endif;

!-----------------------------------------------------------------------
! Causes the opening of openable objects
! Useable only by player
!-----------------------------------------------------------------------

Spell pardem_spell "pardem"
with
   name "pardem",
   purpose "apre oggetti chiusi",
   description "Lanciate questo incantesimo su un oggetto, e questo
      verr@`a aperto. Funziona anche su oggetti chiusi a chiave, o
      su oggetti chiusi magicamente, se il potere dell'incantesimo
      @`e debole.",

   fire [target;
      print "Una luce violetta circonda ", (the) target,
      " che trema sempre pi@`u forte";
      if (~~( target has openable) )
      ", ma semplicemente @`e impossibile aprirlo .";

      if (target has open) ". Ma siccome @`e gi@`a apert", (gan) target,
         "l'incantesimo non ha effetto.";

      give target open;
      give target ~locked;

      ". Dopo una pausa che sembra eterna, di colpo ", (the) target,
         " si apre rumorosamente.";
   ];

!-----------------------------------------------------------------------
! Causes the repair of an object issuing it the SPELL_REPAIR message
! Useable only by player
!-----------------------------------------------------------------------
Spell bendit_spell "bendit"
with
   name "bendit",
   purpose "ripara oggetti danneggiati",
   description "Questo dovrebbe riparare oggetti danneggiato.",

   fire [target;
      print (The) target, " viene circondato da un'aura bianca. ";
      if (~~AffectTarget(target,SPELL_REPAIR))
         " Ma siccome l'oggetto @`e in perffetto stato, nulla accade.";
   ];

!-----------------------------------------------------------------------
! Prevent a location to become a fight_arena, effectively making impossible
! to fight in it.
! Useable only by player
!-----------------------------------------------------------------------
Spell gudil_spell
   with
   name "gudil",
   power 5,
   casting_time 0,
   casting_cost 5,
   purpose "pacifica un luogo, cos@`i che nessun combattimento possa avvenirvi",
   description "Con questo incantesimo, il luogo nel quale si trova il
      mago viene santificato; in questo modo sar@`a impossibile combattervi.
      Ma se lanciato in un luogo gi@`a sacro, l'incantesimo distrugger@`a
      l'aura di protezione e i combattimento potranno di nuovo tenervisi.",

   fire_to_all [loc;
      if (loc ~= nothing) {
         if (loc == thedark) loc = real_location;
         if (loc has absent) {
            give loc ~absent;
            if (location == loc)
               "Il potere della magia che proteggeva questo luogo @`e stato cancellato.";
         }
         else {
            give loc absent;
            if (location == loc)
            "Il potere della magia protegger@`a questo luogo, e gli intenti
            malevoli saranno banditi.";
         }
      }
   ];

!-----------------------------------------------------------------------
! Causes a golem to be summoned in the same room of the player
! --- Required Magical objects to be linked in
! Useable only by player
!-----------------------------------------------------------------------

Ifdef golem;
Spell enjur_spell
with
   name "enjur",
   casting_time 6,
   cost 8,
   power 6,
   purpose "evoca un possente golem che ti difender@`a",
   description [;
      "Questo incantesimo evoca una statua di pietra animata
      (golem) che obbedir@`a a semplici ordini. Ti difender@`a
      in battaglia, o attaccher@`a chi gli sar@`a comandato, 
      e far@`a del suo meglio per soddisfarti. Ma il golem non
      @`e molto intelligente, quindi sii cauto quando gli 
      impartisci degli ordini. E ricorda, puoi evocare
      solo un golem alla volta.";
   ],
   fire_to_all[;
      if (parent(golem) ~= nothing) {
         "Il golem @`e sempre ai tuoi ordini. Non puoi evocarne un altro.";
      }
      ! conjure the golem
      golem.health = golem.max_health;
      golem.alliance = player.alliance;
      move golem to location;
      "Una luce bianca si focalizza di fronte a te. La terra trema,
      e dal profondo delle rocce sotterranee emerge una figura
      imponente.^Dopo poco, tutto @`e finito, ma un possente golem
      @`e in piedi di fronte a te.";
   ];
Endif;

! end of MFS_StandarSpells.h
