! MDIF versione alpha 0.2 di Ignazio Di Napoli
! Versione Modificata il 21/12/2006 da Marco Falcinelli.


!*************************************************************************!
! Costanti interne
Constant BUFFER_LEN = 120;	! lunghezza dell'array buffer
Constant WORDS_LEN  = 10;	! lunghezza dell'array words
CONSTANT WORD_SIZE  = 2;	! numero di byte per parola (al momento supporto solo per Z-machine)

!*************************************************************************!
! Attributi e propriet
Attribute scenery;	! oggetto di scena, non compare nelle descrizioni  

Attribute hidden;	! oggetto nascosto, non compare nelle descrizioni, al momento e' posseduto solo
			! dall'oggetto ME (giocatore)

Attribute showchildren;	! la descrizione dell'oggetto mostra un elenco degli oggetti che contiene, e'
			! posseduto dalle locazioni e dall'oggetto ME (giocatore)

Attribute showName;	! prima della descrizione dell'oggetto viene visualizzato il nome dello stesso
			! in grassetto. E' posseduto dalle locazioni.

Attribute seen;		! viene attribuito agli oggetti una volta che sono stati esaminati

Attribute absent;	! impedisce lo spostarsi degli oggetti con proprieta' found_in, dare l'attributo
			! absent non fa sparire l'oggetto, ma gli impedisce solo di spostarsi
			! in altre locazioni attraverso la proprieta' found_in. Se si vuole eliminare
			! del tutto un oggetto e' necessario dopo avergli dato l'attributo absent
			! operare una istruzione remove

Attribute light;	! rende l'oggetto una sorgente di luce. Di default ogni locazione ha questo
			! attributo.

Property  directions;	! in una locazione, indica i collegamenti con le altre locazioni nelle direzioni
			! cardinali o attraverso vocaboli appositamente definiti. Esempio:
			! ...	
			!	directions	'north' 'bedroom' 0 bed_room
			!			'south' 'kitchen' 0 kitchen_room
			!			'east'  'street' 'out' 0 street_room,
			!	...
			! le direzioni permesse sono quelle cardinali, la funzione LanguageToInformese
			! contiene tutti i sinonimi che vengono accettati automaticamente.
			! le direzione permesse sono quelle contenute nell'oggetto di default obj_directions

Property  useWith;	! indica le interazioni possibili con l'oggetto o tra un oggetto e l'altro,
			! accetta un parametro chiamato 'other' che individua un eventuale secondo oggetto.
			! nel caso la proprieta' non sia presente o ritorni false, un tentativo di
			! interazione con l'oggetto stampera' la sua descrizione.

Property  enter;	! in una locazione, viene richiamata nel momento in cui si entra nella stessa

Property  exit;		! in una locazione, viene richiamata nel momento in cui si esce dalla stessa

Property  tag;		! proprieta' generica

Property  beforechildrenList;	! viene invocata prima di stampare l'elenco degli oggetti figli dell'oggetto
				! che la possiede. Utile per personalizzare il messaggio di introduzione.
				! ad esempio invece del "Puoi vedere" si potrebbe personalizzare
				! il messaggio in "Galleggiano nel vuoto accanto a te:"

Property  think;	! viene invocata ad ogni turno a prescindere dalla posizione dell'oggetto

Property  nextThink;	! flag utile per attivare la proprieta' think dopo un certo numero di turni o all'
			! avverarsi di un determinato evento.

Property  found_in;	! permette di avere lo stesso oggetto in diverse locazioni. Non vengono creati
			! piu' oggetti ma e' lo stesso che viene spostato nella locazione quando vi
			! e' presente il giocatore (il controllo viene fatto ad ogni turno). La
			! proprieta' viene disattivata se l'oggetto possiede l'attributo absent
			! Es. found_in locazione1 locazione2,

Property  describe;	! permette di aggiungere la descrizione di un oggetto in coda a quella della
			! locazione che lo contiene (evitando che compaia nell'elenco degli oggetti 
			! dopo "puoi vedere:")

Property  cant_go;	! permette di personalizzare il messaggio per i movimenti non disponibili 
			! nella locazione. I movimenti intercettati sono quelli non previsti dalla locazione
			! ma presenti nell'oggetto di defualt obj_directions

Property  scenic;	! permette di definire in una locazione (quindi non per tutti gli oggetti)
			! una descrizione per i vocaboli di scenografia  specificati nella propriet. 
			! Evitando il lavoro di definirli tutti come oggetti a se stanti. 
			! Ad esempio in una ipotetica locazione bosco:
			! ...
			!	scenic	'alberi' 'albero' 0 "Faggi alti svettano verso il cielo"
			!		'foglie' 0 "Un tappeto di foglie si stende sul terreno",
			! ...
			! la proprieta' puo' contenere fino a 36 elementi.

Property  invent;	! permette di modificare il nome dell'oggetto quando compare nella lista 
			! dell'inventario del giocatore.
Property link_points 0;	! proprieta' per la libreria miniNpc, necessaria per il calcolo di percorsi 
			! per gli spostamenti dei personaggi non giocatori
Property posDir ;	! proprieta' usata dalla libreria miniNpc, per gli spostamenti dei personaggi
			! non giocatori.
!Property NoTalk;	! il comando LipsCmd controlla la presenza di questa proprieta' per vedere
			! se l'autore ha predisposto un messaggio diverso da quello di default
			! per gli oggetti non parlanti, se il giocatore tenta un dialogo con loro.

!*************************************************************************!
! Variabili Globali utili dall'esterno
Global player = me;		! player individua l'oggetto giocatore

Global location = DefaultRoom;	! contiene la locazione dove si trova il giocatore

Global real_location;		! contiene la locazione reale del giocatore se questo si trova nella
				! locazione di default Darkness (oscuruta')

Global now = 0;		! conteggio di tutte le azioni

Global action_inv = 0;	! flag, si attiva quando il giocatore chiede l'inventario, evita la stampa della
			! descrizione del giocatore prima dell'elenco degli oggetti che possiede.

Global usefullTurns;	! variabile dei turni "produttivi", conteggia i turni escludendo le azioni
			! di sistema e quelle che producono un errore. Viene visualizzata nella statuline

Global no_turn;		! flag per non contare nella variabile usefullTurns le azioni di sistema e i 
			! comandi sbagliati. Si attiva dopo i comandi di sistema o i messaggi di errore.

Global score;		! variabile che conta i punti conquistati dal giocatore.

Global invisible_status;	! flag per eliminare la status line, viene attivata automaticamente
				! se lo schermo e' troppo piccolo

Global v_rec;	! flag per sapere se si  in modalit registrazione o meno


! Variabili Globali e array interni
Array buffer -> BUFFER_LEN + 3;		! contiene gli input di testo del giocatore
Array words buffer 4 * WORDS_LEN + 2;	! trae da buffer le parole (se presenti nel dizionario

!*************************************************************************!
! CLASSI E OGGETTI PREDEFINITI
! classe per definire le locazioni
Class Room
	with	description "UNDER COSTRUCTION",
		posDir 0 0 0 0 0 0 0 0 0 0 0 0, !possibili direzioni per estensione miniNpc.h
		link_points 0, !per tracciare i percorsi degli npc (estensione miniNpc.h)
	has showName showchildren light;

! Oggetti predefiniti
! Locazione di partenza se non diversamente previsto nella routine Init() dall'autore
Room DefaultRoom "Room"
    with description "Room, drescription: lorem ipsum lorem...";

! Locazione Buia, il giocatore vi viene trasportato ogni volta che non e' presente una
! sorgente di luce. La locazione reale viene registrata nella variabile real_location
! le direzioni vengono dirottate sulla proprieta' directions della locazione reale.
Room darkness "Darkness"
	with description "It's pitch dark.", 
	has ~light;

! questo oggetto definisce nel dizionario le possibili direzioni disponibili nel gioco.
! i sinonimi alle direzioni vengono definiti nella routine LanguageToInformese
Object obj_directions "directions" !oggetto delle direzioni (mette nel dizionario tutte le direzioni possibili)
    with description "Possible directions",
	directions 
            'south'	0 ""
            'west'	0 ""
            'north'	0 ""
	    'east'	0 ""
	    'up'	0 ""
	    'down'	0 ""
	    'in'	0 ""
	    'out'	0 ""
	    ;

! oggetto giocatore
Object -> Me "@00"	! @00  la stringa che contiene il nome del PG, e va impostata in INIT(). 
			! Default="you" 
    with name 'blank.' 'blank.' 'blank.' 'blank.' 'you' 'me' 'te' 'yourself', 
		! i nomi "in bianco" possono essere definiti/modificati a cura dell'autore:
		! attraverso una istruzione del tipo:
		! player.&name-->0='Giorgio'; player.&name-->1='Giorgino'; etc...
	 description [;print"";rtrue;], 
		! descrizione vuota, va definita per permettere all'autore di modificarla come crede
	 cambio,	
		! propriet cambio, utile a controllare quale personaggio si impersona in caso 
		! di cambi dello stesso
	 beforechildrenList [; print (string)LibMessages(7);]
		! contiene il messaggio da stampare prima dell'inventario
    has hidden showchildren;


!*************************************************************************!
! Inclusione della libreria per gestire le conversazioni a sclta multipla
! le istruzioni sono presenti nel relativo file.
Include ">minitalk.h";

!*************************************************************************!
! COMANDI PREDEFINITI:
! ogni comando e' rappresentato da un oggetto.

Object QuitCmd "" Me  with name 'quit' 'q//',
         useWith [other;
		no_turn=true; ! non conteggiare questa azione come turno di gioco
		if (other == nothing) {
			print (string) LibMessages(8); 
			if (YesOrNo()) @quit; ! richiesta di conferma
			else rtrue;
		}
		rfalse;
	 ],
    has  scenery;

Object SaveCmd "" Me  with name 'save' 'salva',
         useWith [other;
	      no_turn=true; ! non conteggiare questa azione come turno di gioco
              if (other == nothing) {
			save SaveError; print (string) LibMessages(1); rtrue; !messaggio di errore
			.SaveError; print (string) LibMessages(12); !messaggio di ok
			rtrue;
		}
	      rfalse;],
    has  scenery;

Object RestoreCmd "" Me  with name 'restore' 'carica' 'load',
         useWith [other;
	      no_turn=true; !non conteggiare questa azione come turno di gioco
              if (other == nothing) {
			restore RestoreError; print (string) LibMessages(2); rtrue; !errore
			.RestoreError; print (string) LibMessages(13); !ok
			rtrue;
	      }
              rfalse;],
    has  scenery;

Object WaitCmd "" Me  with name 'z//' 'wait' 'aspetta',
         useWith [other;
	      if (other == nothing) {
			print (string) LibMessages(9);
	      }
              rfalse;],
    has  scenery;

Object DefaultCmd "" Me
    with useWith [obj1;
            if ((obj1 provides usewith && obj1 has seen) || ~~(obj1 provides description)) {
                if (obj1.useWith(nothing) == false)
                    EyesCmd.useWith(obj1);
            } else {
                EyesCmd.useWith(obj1);
            }
         ],
    has  scenery;

Object InventoryCmd "" Me 
    with name 'inv' 'i//' 'inventory' 'inventario',
	 description [;print (string) LibMessages(37);],
         useWith [other;
              if (other == nothing) {
			BeforeInventary();
			action_inv=1;	!variabile globale che si attiva alla richiesta dell'inventario
			EyesCmd.useWith(Player);
			rtrue; 
	      }
              rfalse;
         ],
    has  scenery seen;

Object undo "" ME	
   with  name 'undo', !attenzione l'undo non funziona su tutti gli interpreti
	 UseWith [other i;
		no_turn=true; ! non conteggiare questa azione come turno di gioco
		if (other == nothing) {
			@restore_undo i; 
			if(i==0 && usefullTurns>=0) {
				print(string)LibMessages(15);
				rtrue;
			} 
			else rtrue;
		}
		rfalse;
		],
    has  scenery;

Object obj_restart "" ME	
   with  name 'restart' 'ricomincia', ! per ricominciare la partita
	 UseWith [other;
		no_turn=true; ! non conteggiare questa azione come turno di gioco
		if (other == nothing) {
			print (string) LibMessages(14); 
			if (YesOrNo()) @restart; 
			else rtrue;
		}
		rfalse;
		],
    has  scenery;

Object EyesCmd "" Me
    with name 'x//' 'l//' 'look' 'examine' 'guarda' 'esamina',
         useWith [obj child_Obj first number_of_child; 
	    if (obj) {
		 if (obj.useWith(self) == false) {				    
			first = true;
			if (obj has showName) {
				style bold;
				print "^", (name)obj, "^";
				style roman;
			}
			if (action_inv==0) {
			!se il comando che ha portato qui e' la richiesta di inventario (action_inv==1
			!variabile impostata dal comando "inventario")
			!allora non stampare la descrizione dell'oggetto (personaggio).
				if (obj provides description) {
					PrintOrCommand (obj.description);
					objectloop (child_Obj in obj && child_Obj provides describe) {
						print " ";
						PrintOrCommand(child_Obj.describe);
					}
					print "^";
				}
			}
			action_inv=0; !resetta la variabile
			! Se l'oggetto e' il giocatore stampiamo l'inventario:
			if (obj has showchildren && obj==player) {
				number_of_child=children(obj);	!quanti figli ha l'oggetto
				!da questo numero togliamo quelli non "visibili o nascosti"
				objectloop (child_Obj in obj && (child_Obj has scenery || child_Obj has hidden)) {
					number_of_child--;
				}
				objectloop (child_Obj in obj && child_Obj hasnt scenery && child_Obj hasnt hidden) {
					if (first) {
						first = false;
						if (obj provides beforechildrenList)
							PrintOrCommand (obj.beforechildrenList);
						else print (string)LibMessages(3);
					}
					#ifdef LIST_STYLE;	! diamo la possibilit all'autore di 
								! definire o meno una costante se 
								! preferisce gli oggetti in colonna 
								! o in riga
					if (child_Obj provides invent) {
						print "  ";
						PrintOrCommand(child_Obj.invent);
						print "^";
					} else print "  ", (name)child_Obj, "^"; ! oggetti messi in colonna
					#ifnot;
					! gli oggetti in riga:
					print "  ";
					if (child_Obj provides invent) PrintOrCommand(child_Obj.invent);
					else print (name)child_Obj;
					if (number_of_child>1)print","; ! se ci sono altri figli,
					if (number_of_child==1)print".^"; ! ultimo figlio
					number_of_child--; !aggiorna conto dei figli rimanenti.
					#endif;
				}
			}
			! Se invece l'oggetto non e' il giocatore:
			! (probabilmente e' una locazione)
			else if (obj has showchildren && obj~=player) {
				number_of_child=children(obj);	! quanti figli ha l'oggetto
				objectloop (child_Obj in obj && (child_Obj has scenery || child_Obj has hidden || child_Obj provides describe)) {
					number_of_child--;
				}
				objectloop (child_Obj in obj && child_Obj hasnt scenery && child_Obj hasnt hidden && ~~(child_Obj provides describe)) {
					if (first) {
						first = false;
						if (obj provides beforechildrenList)
							PrintOrCommand (obj.beforechildrenList);
						else print (string)LibMessages(3);
					}
					#ifdef LIST_STYLE; !definisci questa costate se vuoi gli oggetti
					print "  ", (name)child_Obj, "^"; ! messi in colonna
					#ifnot;
					print "  ", (name)child_Obj;
					if (number_of_child>1)print",";   ! se ci sono altri figli
					if (number_of_child==1)print".^"; ! ultimo figlio
					number_of_child--; ! aggiorna conto dei figli rimanenti.
					#endif;
				}
			}
			give obj seen; ! l'oggetto e' stato esaminato
			return true;
	        }
	    }
	    else print_ret (string) LibMessages(36);
        ],
    has  scenery;

Object HandsCmd "" Me
    with name 'use' 'u//' 'usa',
         useWith [other;
             if (other) return other.useWith(nothing);
	     else print_ret (string) LibMessages(26);
         ],
    has  scenery;

Object LipsCmd "" Me
   with name 'talk' 'ask' 'tell' 't//' 'parla' 'chiedi',
	useWith [other;
		if (other==nothing or HandsCmd or self or EyesCmd) print_ret (string) LibMessages (40);
		else if (other provides InitQuips) {other.StartTalk();rtrue;}
		else if (other provides NoTalk) {other.NoTalk();rtrue;}
		else print_ret (name) other, (string) LibMessages (41);
	],
   has scenery;

! COMANDI DI DEBUG - da togliere in una avventura completata
Object recOn "" ME	
   with	 name 'recon',  ! per registrare i comandi
	 UseWith [other;
		no_turn=true; ! non conteggiare questa azione come turno di gioco
		if (other == nothing) {rec_on_command();rtrue;}
		rfalse;
		],
    has  scenery;

Object recOff "" ME	
   with  name 'recoff', ! per fine registrazione comandi
	 UseWith [other;
		no_turn=true; ! non conteggiare questa azione come turno di gioco
		if (other == nothing) {rec_off_command();rtrue;}
		rfalse;
		],
    has  scenery;

Object rePlay "" ME	
   with  name 'play' 'replay', ! per avviare il replay dei comandi
	 UseWith [other;
		no_turn=true; !non conteggiare questa azione come turno di gioco
		if (other == nothing) {play_command();rtrue;}
		rfalse;
		],
    has  scenery;


!*************************************************************************!
! Libreria di messaggi (in italiano e in inglese)
[LibMessages index;
  #ifdef ITALIANO; !costante da definire per i giochi in italia
    switch (index) {
        0: return "^> ";
        1: return "Errore durante il salvataggio.^";
        2: return "Errore durante il caricamento.^";
        3: return "Inoltre, puoi vedere:^";
        4: return "Non ho capito.^";
        5: return "Errore: troppe parole.^";
        6: return "Scegli cosa dire: ";
        7: return "Stai portando:^";
	8: return "^Vuoi uscire dal gioco?(S/N)^";
	9: return "Il tempo passa...^^";
	10: return "^Per favore premi S (Si) o N (No): ^";
	11: return "^   [more]";
	12: return "Salvataggio riuscito^";
	13: return "Caricamento riuscito^";
	14: return "Vuoi ricominciare?(S/N)^";
	15: return "UNDO FALLITO!^";
	16: return "ERRORE NEL SALVATAGGIO DELL'UNDO, CONTATTA L'AUTORE DEL GIOCO^";
	17: return "registrazione comandi attiva^";
	18: return "La registrazione  gi attiva.^";
	19: return "registrazione comandi disattiva^";
	20: return "La registrazione  gi disattivata^";
	21: return "Esecuzione dei comandi registrati completata^";
	22: return "T: "; !turni
	23: return "P: "; !punti
	24: return "Turni: ";
	25: return "Punti: ";
	26: return "Non ho capito su cosa vuoi agire.";
	27: return "Non conosco la parola ~";
	28: return "Non puoi andare in quella direzione.^";
	29: return "Non  possibile esaminare ~";
	30: return "~ in questo momento.^";
	31: return "Non  possibile usare ~";
	32: return " con altre cose.^";
	33: return "~ e ~";
	34: return "~ assieme.^";
	35: return "~ non  disponibile in questo momento.^";
	36: return "Non ho capito cosa vuoi esaminare.";
	37: return "E' la tua fidata sacca.^";
	38: return "Non puoi esaminare il verbo ~esaminare~^";
	39: return "Vuoi esaminare o agire?^";
	40: return "Con chi vuoi parlare?";
	41: return " non ha nulla da dire.";
	42: return "Non puoi parlare con ~";
   #ifnot; ! l'inglese e' la lingua di default
     switch (index) {
        0: return "^> ";
        1: return "Saving Error^";
        2: return "Loading Error^";
        3: return "You can see:^";
        4: return "I didn't understand.^";
        5: return "Error: too many words...^";
        6: return "What would you like to say?^";
        7: return "You are carrying:^";
	8: return "^Are you sure you want to quit?(Y/N)";
	9: return "Time passes...^^";
	10: return "^Please answer Y (yes) or N (no): ^";
	11: return "^   [more]";
	12: return "File Saved.^";
	13: return "File Loaded.^";
	14: return "Are you sure you want to restart?(Y/N)^";
	15: return "UNDO FAILED!^";
	16: return "Error saviving the undo position. Please contact the game's author.^";
	17: return "Recording on^";
	18: return "Recording is already on.^";
	19: return "Recording off^";
	20: return "Recording is already off.^";
	21: return "Replaying complete.^";
	22: return "T: "; !turns
	23: return "P: "; !points
	24: return "Turns: ";
	25: return "Points: ";
	26: return "Don't understand what you want use.";
	27: return "Don't know the word ~";
	28: return "You can't go that way.^";
	29: return "Can't examine ~";
	30: return "~ now.^";
	31: return "Can't use ~";
	32: return " with other things.^";
	33: return "~ and ~";
	34: return "~ together.^";
	35: return "~ isn't avaible in this moment.^";
	36: return "Don't understand what you want examine.";
	37: return "It's your trusty sack.^";
	38: return "You can't examine the verb ~examine~^";
	39: return "Do you want to examine o use?^";
	40: return "Don't understand who you want to talk to...";
	41: return " has nothing to say.";
	42: return "Can't talk to ~";
   #endif;
    }
];

!*************************************************************************!
! Funzioni interne

! Per ottenere l'oggetto dal vocabolo di dizionario: 
[ObjectFromDictName dictname loc x i t;
    if (dictname == 0)
        return nothing;
    objectloop (x in loc && x provides name){
        for (i = 0: i < x.#name / WORD_SIZE: i++) {
            if (x.&name-->i == dictname) {
                return x;
            }
        }
    }
    objectloop (x in loc && children(x) > 0) {
        t = ObjectFromDictName(dictname, x);
        if (t ~= nothing)
            return t;
    }
    return nothing;
];

! per stampare stringhe, richiamare routine o proprieta', spostarsi tra le locazioni etc...
[PrintOrCommand t;
    switch (metaclass(t)) {
        String:
            print (string)t;
            rtrue;
        Object:
            if (location provides exit)
                PrintOrCommand (location.exit);
            move player to t;

	    if (t provides enter)
		PrintOrCommand (t.enter);
            
	    NewRoom(); !entrypoint
	    
	    rtrue;
        Routine:
            t.call();
            rtrue;
        default: 
            rfalse;
    }    
];

! controllare se la parola di dizionario trovata dal parser e' una direzione
[CheckForDirections dictname obj i; 
    if (~~(obj provides directions)) rfalse;
    
    for (i = 0: i < obj.#directions / WORD_SIZE: i++) {
        if (dictname == obj.&directions-->i) {
            do { 
                i++;
            } until (obj.&directions-->i == 0);            
            i++;

            return PrintOrCommand(obj.&directions-->i);
        }
    }
    
    rfalse;
];

! controllare se la parola di dizionario trovata dal parser e' un vocabolo di scena
[CheckForScenic dictname obj i; !dictname e' la parola inserita dal giocatore, obj la locazione dove si trova
    if (~~(obj provides scenic)) rfalse;
    for (i = 0: i < obj.#scenic / WORD_SIZE: i++) {
        if (dictname == obj.&scenic-->i) {
            do { 
                i++;
            } until (obj.&scenic-->i == 0);            
            i++;

            return PrintOrCommand(obj.&scenic-->i);
        }
    }
    
    rfalse;
];

!controlla se c'e' una sorgente di luce nella locazione
[CheckForLight t lightflag;
	lightflag=false;
	if (parent(player) has light) lightflag=true;
	else if (real_location && parent(player)==darkness && real_location has light) lightflag=true;
	else {
		objectloop (t has light && ~~(t ofclass room)) {
			if (IndirectlyContains(parent(player),t)) lightflag=true;
			if (IndirectlyContains(real_location,t)) lightflag=true;
		}
	}
	if (lightflag==false) {
		location=parent(player);
		if (parent(player)~=darkness) real_location=parent(player);
		location=darkness;
		move player to darkness;
	}
	else if (lightflag && parent(player)~=darkness) {
		location = parent(player);
		real_location = parent(player);
	}
	else if (lightflag && parent(player)==darkness) {
		location=real_location;
		move player to real_location;
	}
	!se voglio monitorare i cambiamenti di real_location
	!print "REAL LOCATION ", (name)real_location;
];

!stampa della "w"-esima parola dell'input del giocatore ovvero nell'array Buffer.
[PrintWordAddress w x lun pos;
	lun = words->(4 * w);
        pos = words->(4 * w + 1);
	for (x=0:x<lun:x++) print (char) buffer->(pos+x);	
];


!*************************************************************************!
! Il Cuore della libreria
[Main
    t w length position dict obj1 obj2 dict1 dict2 ret i;
    ClearScreen(); ! pulizia dello schermo all'inizio del gioco.
    now = 0;	   ! inizializzazione dei turni
    objectloop (t ofclass Character) { t.InitQuips(); } !inizializzazione delle conversazioni
    string 00 "you";	! nome predefinito del giocatore
    Init();		! entrypoint per l'inizio del gioco, OBBLIGATORIA
			! deve definire quanto meno la locazione iniziale del gioco
			! location=prima_locazione;
    move player to location;
    real_location=location;
    
    !inizia il gioco
    for (::) {
	! gestione della luce, se c'e' una sorgente luminosa tutto bene altrimenti si viene trasportati
	! nella locazione buia di default Darkness
	GamePreRoutine();
	CheckForLight();	!se si vuole eliminare tale gestione commentare questa riga
	
	!location=parent(player);	!se si vuole escludere la gestione della luce togliere il commento
					!a questa istruzione

	!implementazione della propriet found_in e dell'attributo absent
	objectloop (t provides found_in && t hasnt absent) {
		for (w=0:w<t.#found_in / WORD_SIZE:w++) {
			if (location==t.&found_in-->w) move t to location;
		}
	}
	
	! Salvataggio della posizione per l'undo:
	@save_undo i; if(i==-1){i=0; print_ret (string) LibMessages(16);}

	! Statusline
        DrawStatusLine(); 
	! Stampa della descrizione della locazione
	EyesCmd.useWith(location);
        
	! Controllo delle proprieta' think per ogni oggetto
        objectloop (t provides think) {
            if ((t provides nextThink && now >= t.nextThink && t.nextThink ~= 0) 
			|| ~~(t provides nextThink)) {
                
		if (t provides nextThink) t.nextThink = 0;
                
		t.think();
            }
        }
    
	BeforePrompt();			!entrypoint
        print (string)LibMessages(0);	!stampa del prompt
        
	buffer->0 = BUFFER_LEN;		!definizione delle array
        words->0  = WORDS_LEN;		
        
        dict1 = nothing;		!reset dei vocabili
        dict2 = nothing;
        
        read buffer words;		! richiesta dell'input
	LanguageToInformese();		! eventuale correzione dell'input
	BeforeParsing();		!entrypoint
        
	for (w = 1: w <= words->1: w++) {	!parsing
            length = words->(4 * w);
            position = words->(4 * w + 1);
            dict = words-->(w * 2 - 1);
	    if (w == 1)
                dict1 = dict;
            else if (w == 2)
                dict2 = dict;
	    else {				!sono accettati solo due comandi
                print (string)LibMessages(5);
	    	no_turn=true;
                dict1 = nothing;
            }
	}

	if (dict1 ~= nothing) {
            obj1 = ObjectFromDictName(dict1, location);
            if (obj1 ~= nothing) {
                if (dict2 == nothing) {
                    DefaultCmd.useWith(obj1);
                } else {
                    obj2 = ObjectFromDictName(dict2, location);
                    if (obj2 == nothing){
                        !print (string)LibMessages(4); 
			!ha capito il primo oggetto, il secondo nome  nel dizionario ma non  un oggetto.
			if (obj1==EyesCmd) {
				ret=CheckForScenic(dict2, location); !il giocatore ha digitato "esamina oggetto di scena"?
				if (ret==false) {
					print (string) LibMessages(29);
					PrintWordAddress(2);
					print (string) LibMessages(30);
					no_turn=true;
				}
			}
			else if (obj1==HandsCmd) {
				ret=CheckForScenic(dict2, location); !il giocatore ha digitato "usa oggetto di scena"?
				if (ret==false) {
					print (string) LibMessages(31);
					PrintWordAddress(2);
					print "~.^";
					no_turn=true;
				}
			}
			else if (obj1==LipsCmd) {
				ret=CheckForScenic(dict2, location); !il giocatore ha digitato "parla a oggetto di scena"?
				if (ret==false) {
					print (string) LibMessages(42);
					PrintWordAddress(2);
					print "~.^";
					no_turn=true;
				}
			}
			else { !il giocatore ha chiesto di usare assieme due oggetti che l'autore non ha previsto.
				print (string) LibMessages(31);
				PrintWordAddress(2);
				print (string) LibMessages(32);
				no_turn=true;
			}
		    }
                    else {
			!evitiamo il bug su istruzioni con due oggetti/Verbi tipo "x x"
			if (obj1==EyesCmd && obj2==EyesCmd) print (string) LibMessages(38);
			else if ((obj1==EyesCmd && obj2==HandsCmd)||(obj1==HandsCmd && obj2==EyesCmd)) print (string) LibMessages(39);
                        else {
				ret = obj1.useWith(obj2);
				if (ret == false)
				    ret = obj2.useWith(obj1);
				if (ret == false) { 
				    !ha capito entrambi gli oggetti ma non possono essere usati assieme.
				    print (string) LibMessages(31);
				    PrintWordAddress(1);
				    print (string) LibMessages(33);
				    PrintWordAddress(2);
				    print (string) LibMessages(34);
				    no_turn=true;
				    !print (string)LibMessages(4); 
				}
			}
                    }
                }
            } else {
		!controllo per la sorgenete di luce, se si vuole eliminare, commentare la prima riga
		!e togliere il primo else:
	        if (location==darkness) ret= CheckForDirections(dict1, real_location);
		else ret = CheckForDirections(dict1, location);
                if (ret == false) {
		    ret = CheckForDirections(dict1, obj_directions);
		    if (ret) {
			!il giocatore ha digitato una direzione non disponibile nella locazione.
			if (location provides cant_go) PrintOrCommand(location.cant_go);
			else print (string) LibMessages (28);
			no_turn=true;
		    } 
                    else {
			ret=CheckForScenic(dict1, location); !il giocatore ha digitato un oggetto di scena?
			if (ret==false) {
				!print (string)LibMessages(4); 
				!la prima parola  nel dizionario del gioco, ma non chiamabile al momento.
				no_turn=true;
				print "~";
				PrintWordAddress(1);
				print (string) LibMessages(35);
			}
		    }
		    
                }
            }
        } else {
	    print (string)LibMessages(27); !la prima parola  al di fuori del dizionario.
	    PrintWordAddress(1);
	    print "~.^";
	    no_turn=true;
        }
        
        now++;	!incremento dei turni
	if(no_turn==false) {
		usefullTurns ++; !incremento del numero di azioni valide e non di sistema.
		TimePasses(); !entrypoint
	} 
	no_turn=false; !reset del controllo sul conteggio delle azioni valide.
    }
];


