! wtalk.h  Menu driven conversational library
!          Paolo Lucchesi, rel. 1.4.9, 2008-06-24
!          Inspired by the conversational menus in Emily Short's games
!          Pytho's Mask and City of Secrets
!          Thanks to Alessandro Schillaci for dynamic glulx talk window code
!
! ------------------------------------------------------------------------------
!
! This file is free software, it can be freely used, copied, distribuited and
! modified. Any modified or unmodified copy of this file distribuited as source
! code must acknoledge the original author and must be freely usable, copyable,
! distributable and modifiable.
! There's absolutely no condition or binding on the distribution as compiled
! binary file.
! This software is distribuited as-is, with NO WARRANTY. The author cannot be
! held as responsable for any damage due directly or indirectly to this file.
!
! ------------------------------------------------------------------------------
!
! Library 6/11 or greater needed
!
! How to use:
! Put in your main file the directive
!     include "wtalk";
! three times: just before including parser.h, right after including verblib.h
! and after including grammar.h
!
!
! Each non-player character should belong to NPC class (which has already the
! animate attribute set), and have a conversor property pointing to an object
! which is the conversation controller. All the dialog options available
! should be children of this conversor, so you can add and subract dialog
! options by moving them into the conversor or removing them.
! Dialog options should belong to Dialog Class. Their short name is the
! question (and the line shown in statusline / menu window), while their
! description property is the answer (and can be a routine).
! Quotation marks are added to the question, but not to the answer.
! Optionally, if the dialog option provides an after property, this property
! is run after showing question and answer.
! Also, if the dialog options provides a before property, this property is
! run before showing question and answer; furthermore, if the before property
! returns true, question and answer are not shown (i.e., the description and
! after properties are not evaluated).
! After a dialog option is choosen, it's removed from the list unless it have
! the static attribut set, and all its children are added to the list.
! Finally, is a dialog object has the locked attribute set, the dialog isn't
! used, nor shown in the statusline / menu window (as if it weren't child of
! the conversor)
!
! The author can begin a conversation with a given character calling
! talkwith(npc, flag), where npc is the character to talk with and flag
! is a true/false variable that states if the conversation should be
! started even if the player is yet talking with someone else.
! Also the stoptalk() routine stops the current conversation (this also
! could be used at the game end).
! You can check if a NPC has still some valid dialog options available
! with talkcheck(npc). The talking() function checks if there's an active
! conversation and, if so, returns the NPC the player is talking with
! (else, it returns 0).
!
! Example
!
! NPC george "George Talker"
!   with name 'george' 'talker',
!   conversor georgetalk,
!   has male proper;
!
! Object georgetalk;
!
! Dialog -> gthello "Hello, Mr. Talker, how are you?"
!   with description "~Good Morning to you, sir.~";
!
! Dialog -> gtwhere "Where are you going?"
!   with description [;
!      move gtbank to georgetalk;
!      "~I'm going to the central bank.~";
!   ];
!
! Dialog  gtbank "But today is saturday, the bank is closed."
!   with description "~Oh. That's right. Thanks a lot.~",
!   has static;
!
!
! Notes:
! In Glulx the dialogs window dynamically appears and disappear at need. If
! you want a static window, define the STATIC_TALKWINDOW (before including
! wtalk.h).
! You can always use TalkWindowOpen(), TalkWindowReopen() (the latter
! redraws the dialog lines) and TalkWindowClose() to manipulate the
! dialogs window.
! Define the TALK_WIN_UFRAME constant to have a line drawn between the main and
! conversation windows (to use with WinGlulxe no-frames configurations).
!
! This extension use the TimePasses() Entry Point (which just calls the 
! dialogline.operate() routine) to react to npc-moving daemons. If you have
! defined your own TimePasses(), insert the dialogline.operate() call in your
! routine and define the TALK_NOTIMEPASSES constant before including wtalk.h,
! or else rename your TimePasses() function in CustomTimePasses().
! 
! If you have a personalized statusline, rename your DrawStatusLine() function
! in CustomDrawStatusLine() and remove the Replace DrawStatusLine directive;
! The CustomDrawStatusLine() function will be suitably called by walk's
! own DrawStatusLine()
!
! WARINING:
! In Glulx, to make the talk window react properly to the SAVE, RESTORE and
! RESTART command, you'll have to add the following line
!    TalkIdentifyGlkObject(phase,type,ref,rock);
! at the end of your IdentifyGlkObject entry point routine. Be sure that such
! line will be executed in every call to IdentifyGlkObject.
! You can define the constant CANT_TALK_ANSWER as a string or as the name of
! a routine to customize what is printed when the player try to talk to 
! someone but there aren't topic left. You can also define it as NULL to
! have no answer at all.
! If you define a WTALK_REMOVE_YESNO constant, then the Yes and No actions
! will be refused (like Tell, Ask, AskFor, Answer and all orders).
! ------------------------------------------------------------------------------

!Global talkwith;

#Ifndef LIBRARY_PARSER;
!First inclusion
message "Using wtalk_it 1.4.9 by Paolo Lucchesi";
#IfDef TARGET_ZCODE;
Replace DrawStatusLine;
#Endif;
Replace TellSub;
Replace AskSub;
Replace AnswerSub;
Replace AskForSub;
#IfNDef WTALK_REMOVE_YESNO;
Replace YesSub;
Replace NoSub;
#EndIf;

Property conversor;

#IfDef TARGET_GLULX;
  Constant GG_TALKWIN_ROCK 297;
  Global gg_talkwin = 0;
#IfDef TALK_WIN_UFRAME;
  Constant GG_TALKWINUF_ROCK 296;
  Global gg_talkwinuf = 0;
#Endif;
#Endif;

#Endif;

!-------------------------

#Ifdef LIBRARY_PARSER;
#Ifndef LIBRARY_GRAMMAR;
!Second inclusion

Global talkinited = false;  

Object "(wtalk)" LibraryExtensions
  with ext_initialise [;
    !StartDaemon(dialogline);
    talkinited = true;
    TalkWindowClose();
  #Ifdef STATIC_TALKWINDOW;
    TalkWindowOpen();
  #Endif;
    rtrue;
];

Stub TalkInit 0; ! Legacy

#IfDef TARGET_GLULX;

[ TalkWindowOpen bg fg;
    if (gg_talkwin == 0) {
        gg_talkwin = glk_window_open(gg_mainwin, (winmethod_Below+winmethod_Fixed), 4, wintype_Textgrid, GG_TALKWIN_ROCK);
    }
    bg = 0; fg = $FFFFFF;
#IfDef TALK_WIN_UFRAME;
    if (glk_style_measure(gg_talkwin, style_Normal, stylehint_TextColor, gg_arguments)) 
        fg = gg_arguments --> 0;
    if (glk_style_measure(gg_talkwin, style_Normal, stylehint_BackColor, gg_arguments)) 
        bg = gg_arguments --> 0;
    if (gg_talkwinuf == 0) {
        gg_talkwinuf = glk_window_open(gg_mainwin, (winmethod_Below+winmethod_Fixed), 4, wintype_Graphics, GG_TALKWINUF_ROCK);
    }    
    glk_window_set_background_color(gg_talkwinuf, bg); 
    glk_window_clear(gg_talkwinuf);        
    glk_window_fill_rect(gg_talkwinuf,fg,1,1,2048,2);
#Endif;
    rtrue;
];

[ TalkWindowReopen;
    if (talkinited) dialogline.display();
];

[ TalkWindowClose;
    if (gg_talkwin) glk_window_close(gg_talkwin, 0);
    gg_talkwin = 0;
#IfDef TALK_WIN_UFRAME;
    if (gg_talkwinuf) glk_window_close(gg_talkwinuf, 0);
    gg_talkwinuf = 0;
#Endif;
    rtrue;    
];

#Endif;

Stub TalkWindowOpen 0;
Stub TalkWindowReopen 0;
Stub TalkWindowClose 0;

#Ifndef TALK_NOTIMEPASSES;
[ TimePasses;
    dialogline.operate();
#Ifdef CustomTimePasses;
    CustomTimePasses();
#Endif;		
];
#Endif;

Object dialogline
  with ndialogs 0, dialog1 0, dialog2 0, dialog3 0,
    talker 0, lastturn false,
    operate [x y;
      self.ndialogs=0; self.dialog1=0; self.dialog2=0; self.dialog3=0;
      if (self.talker) if (~~TestScope(self.talker)) self.talker=0;
      !if (self.talker) if (self.talker notin real_location) { self.talker=0; return; }
      if (self.talker) {
          y=self.talker;
          objectloop (x in y.conversor) {
              if (x ofclass Dialog && x hasnt locked && self.ndialogs<3) {
                  switch(++self.ndialogs) {
                      1: self.dialog1 = x;
                      2: self.dialog2 = x;
                      3: self.dialog3 = x;
                  }
              }
          }
          if (self.ndialogs==0) self.talker=0;
      }
      self.display();
    ],
    display [lines tp i;
      lines = self.ndialogs; if (lines > 3) lines = 3;
      tp = i; ! Just to avoid warnings
#Ifdef TARGET_GLULX;
    #Ifdef STATIC_TALKWINDOW;
      TalkWindowOpen();
    #Endif;
      if (self.ndialogs > 0) {
          TalkWindowOpen();
          if (gg_talkwin) {
              glk_set_window(gg_talkwin);
              glk_window_clear(gg_talkwin);
              glk_window_move_cursor(gg_talkwin,0,0);
              print "(Talking to ", (the) self.talker, ")^";
              for (i=1: i<=lines: i++) {
                  switch(i) { 1: tp=self.dialog1;
                              2: tp=self.dialog2;
                              3: tp=self.dialog3;
                  }
                  glk_window_move_cursor(gg_talkwin,0,i);
                  print " ", i, ") ", (name) tp, "^";
              }
              MainWindow(); ! set_window
          }
      }
      else {
        #Ifdef STATIC_TALKWINDOW;
          if (gg_talkwin) {
              glk_set_window(gg_talkwin);
              glk_window_clear(gg_talkwin);
              MainWindow();
          }
        #Ifnot;
          TalkWindowClose();
        #Endif;
      }
#Endif;
    ];

[ talking;
    if (dialogline.ndialogs > 0) return dialogline.talker;
    rfalse;
];	

#Ifdef TARGET_ZCODE;
Array dlstring buffer 80;

[ DrawStatusLine width posa posb lines i k tp;
![ DrawStatusLine width posa posb;
    posa = posb; ! warning squasher

    #Ifdef TARGET_GLULX;
    ! If we have no status window, we must not try to redraw it.
    if (gg_statuswin == 0)
        return;
    #Endif;

    ! If there is no player location, we shouldn't try to draw status window
    if (location == nothing || parent(player) == nothing)
        return;

    width = ScreenWidth();
#Ifdef TARGET_ZCODE;
    MoveCursor(1, 1);
    if (dialogline.ndialogs <= 0) {
        if (dialogline.lastturn) {
            StatusLineHeight(gg_statuswin_size+4);
            style roman;
            for (i=1: i<=4: i++) {MoveCursor(gg_statuswin_size+i, 1); spaces width; }
            style reverse;
        }
        StatusLineHeight(gg_statuswin_size);
    }
    else StatusLineHeight(gg_statuswin_size+4);
#Ifnot; ! TARGET_GLULX
    StatusLineHeight(gg_statuswin_size);
#Endif;

    MoveCursor(1, 1);

#Ifdef CustomDrawStatusLine;
    CustomDrawStatusLine();
#Ifnot;

    posa = width-26; posb = width-13;
    spaces width;

    MoveCursor(1, 2);
    if (location == thedark) {
        print (name) location;
    }
    else {
        FindVisibilityLevels();
        if (visibility_ceiling == location)
            print (name) location;
        else
            print (The) visibility_ceiling;
    }

#Ifdef TARGET_ZCODE;
  if (dialogline.ndialogs <= 0) {
#Endif;
    if (sys_statusline_flag && width > 53) {
        MoveCursor(1, posa);
        print (string) TIME__TX;
        LanguageTimeOfDay(sline1, sline2);
    }
    else {
        if (width > 66) {
            #Ifndef NO_SCORE;
            MoveCursor(1, posa);
            print (string) SCORE__TX, sline1;
            #Endif;
            MoveCursor(1, posb);
            print (string) MOVES__TX, sline2;
        }
        #Ifndef NO_SCORE;
        if (width > 53 && width <= 66) {
            MoveCursor(1, posb);
            print sline1, "/", sline2;
        }
        #Endif;
    }
#Ifdef TARGET_ZCODE;
  }
#Endif;

#Endif;

#Ifdef TARGET_ZCODE;
    dialogline.lastturn=false;
    if (dialogline.ndialogs > 0) {
       dialogline.lastturn=true;
       dlstring->0 = 78;
       @output_stream 3 dlstring;
       print "(Talking to ", (the) dialogline.talker, ")";
       @output_stream -3;
       k = width - dlstring-->0 - 2;
       MoveCursor(1,k);
       print "(Talking to ", (the) dialogline.talker, ")";

       for (i=2: i<=5: i++) { MoveCursor(i, 1); spaces width; }
       lines = dialogline.ndialogs; if (lines > 3) lines = 3;
       for (i=1: i<=lines: i++) {
           switch(i) { 1: tp=dialogline.dialog1;
                       2: tp=dialogline.dialog2;
                       3: tp=dialogline.dialog3;
                     }
           k=i+gg_statuswin_size; MoveCursor(k, 1); print "  ", i, ") ", (name) tp;
       }
    }
#Endif;

    MainWindow(); ! set_window
];

#Endif;

[ xinternalcanttalk;
  #Ifdef NPerson;
    if (NPerson==FIRST_PERSON) "I don't have anything to say.";
  #Endif;
    "You don't have anything to say.";
];

Default CANT_TALK_ANSWER xinternalcanttalk;

Class NPC
  with before [x;
    TalkTo:
      if (~~(self provides conversor)) rfalse;
      !r = true;
      !for (x = (child(self.conversor)) : x && r : x = sibling(x) ) if (x hasnt locked) r = false;
      if ((~~talkcheck(self)) && CANT_TALK_ANSWER ~= 0 or NULL) { PrintOrRunVar(CANT_TALK_ANSWER); rtrue; }
      dialogline.talker=self;
      x=self.conversor;
      if (x provides before) x.before();
      dialogline.operate();
      rtrue;
  ],
  orders [; talktomsg(); rtrue; ],
has animate;

[ talkcheck obj x;
  if (~~(obj provides conversor)) rfalse;
  for (x = (child(obj.conversor)) : x : x = sibling(x) ) if (x hasnt locked) rtrue;
  rfalse;
];    

[ talkwith obj force x;
    if (dialogline.talker && ~~force) rtrue;
    if (obj == 0 or NULL) {
        dialogline.talker = 0;
        rtrue;
    }
    if (obj ofclass NPC && obj provides conversor) {
        dialogline.talker=obj;
        x=obj.conversor;
        if (x provides before) x.before();
        dialogline.operate();
        rtrue;
    }
    rfalse;
];

[ stoptalk;
    dialogline.talker = 0;
    dialogline.operate();
];

[ talktomsg; "Just use ~Talk to character~."; ];

Class Dialog;

!--- Warning squasher
Object xwtalkxdummyx
   with xproc [; talkwith(); stoptalk(); TalkWindowOpen(); TalkWindowReopen();
                 TalkWindowClose(); TalkInit(); ];

[ TalkToSub;
  #Ifdef NPerson;
    if (noun == player && NPerson==FIRST_PERSON) "Better not to talk to myself.";
  #Endif;
    if (noun == player) "Better not to talk to yourself.";
    print (The) noun;
    if (noun has pluralname) print " don't"; else print " doesn't";
    if (~~(noun ofclass NPC)) " seem capable to talk.";
    " seem interested.";
];

[ Dialog1Sub; DoDialog(1); ];
[ Dialog2Sub; DoDialog(2); ];
[ Dialog3Sub; DoDialog(3); ];

[ DoDialog op tp rval x y;
    if (dialogline.talker) if (~~TestScope(dialogline.talker)) dialogline.talker=0;
  #Ifdef NPerson;
    if (dialogline.talker==0 && NPerson==FIRST_PERSON) "But I'm not talking with anyone, right now...";
  #Endif;
    if (dialogline.talker==0) "You're not talking with anyone, right now.";
    if (op > dialogline.ndialogs) rfalse;
    switch(op) { 1: tp=dialogline.dialog1;
                 2: tp=dialogline.dialog2;
                 3: tp=dialogline.dialog3;
               }
    rval = false;
    if (tp provides before) if (tp.before()) rtrue;
    print "~", (name) tp, "~^";
    if (tp provides description) rval=tp.description();
    if (tp provides after) rval=tp.after();
    x = child(tp);
    while (x) {
        y = sibling(x);
        if (x ofclass Dialog) move x to parent(tp);
        x = y;
    }
    if (tp hasnt static) remove tp;
    return rval;
];

[ TellSub; talktomsg(); ];
[ AskSub; talktomsg(); ];
[ AnswerSub; talktomsg(); ];
[ AskForSub; talktomsg(); ];
#IfNDef WTALK_REMOVE_YESNO;
[ YesSub; talktomsg(); ];
[ NoSub; talktomsg(); ];
#EndIf;

#Ifdef TARGET_GLULX;
[ TalkIdentifyGlkObject phase type ref rock;
    !print "#", phase, "#"; KeyCharPrimitive();
    if (phase == 0) gg_talkwin = 0;
    if (phase == 1 && type == 0 && rock == GG_TALKWIN_ROCK) gg_talkwin = ref;
#IfDef TALK_WIN_UFRAME;
    if (phase == 0) gg_talkwinuf = 0;
    if (phase == 1 && type == 0 && rock == GG_TALKWINUF_ROCK) gg_talkwinuf = ref;
#Endif;
    if (phase == 2) {
        TalkWindowClose();
        if (talkinited) dialogline.display();
    }
    rtrue;
];
#Endif;

#Endif;
#Endif;

!-------------------------------------

#Ifdef LIBRARY_GRAMMAR;
! Third inclusion
Verb 'talk'       * 'to'/'with'/'at' creature     -> TalkTo;

Verb '1'        *                               -> Dialog1;
Verb '2'        *                               -> Dialog2;
Verb '3'        *                               -> Dialog3;
#Endif;
