! wtalk_it.h  Sistema di conversazioni a menu'
!             Paolo Lucchesi, rel. 1.4.9, 2008-06-24
!             Inspirato dai menu conversazionali nei giochi
!             Pytho's Mask and City of Secrets, di Emily Short
!             Grazie a Alessandro Schillaci per il codice delle finestre glulx
!
! ------------------------------------------------------------------------------
!
! Questo file e' software libero, puo' essere liberamente usato, copiato,
! distribuito e modificato. Qualsiasi versione, modificata o meno, di questo
! file distribuita sotto forma di codice sorgente deve contenere riferimento
! all'autore originale e deve essere liberamente usabile, copiabile,
! distribuibile e modificabile.
! Non e' posta alcuna condizione alla distribuzione del file come codice
! compilato.
! Questo file e' distribuito senza alcuna garanzia. L'autore non puo'
! essere ritenuto responsabile in alcun modo di danni provocati
! direttamente o indirettamente da questo file.
!
! ------------------------------------------------------------------------------
!
! Richiesta libreria 6/11 o successive, e Infit 2.5 o successive
!
! Come usare questa estensione
! Inserite nel vostro file principale la direttiva
!     include "wtalk_it";
! tre volte: appena prima di includere parser.h, subito dopo aver incluso
! verblib.h e replace.h e dopo aver incluso italiang.h
!
! Ogni personaggio-non-giocatore deve appartenere alla classe NPC (che ha gia'
! l'attributo animate settato), e deve avere una proprieta' conversor che punta
! ad un oggetto che e' il controllore della concersazione. Tutte le opzioni di
! dialogo possibili devono essere figli di questo conversor, cosi' potrete
! aggiungere o sottrarre opzioni di dialogo semplicemente muovendole nel
! conversor (move ... to ...) o rimuovendole (remove ...).
! Le opzioni di dialogo devono appartenere alla classe Dialog. Il loro
! short_name (quello definito nella prima riga, per intenderci) contiene la
! domanda (e cio' che viene mostrato nella statusline o nella finestra dei
! menu'), mentre la proprieta' description contiene la risposta (e puo' essere
! una routine).
! Le virgolette vengono aggiunte alla domanda, ma non alla risposta.
! Opzionalmente, se l'opzione di dialogo ha una proprieta' after, questa viene
! eseguita dopo la visualizzazione del dialogo.
! Inoltre, se l'opzione di dialogo ha una proprieta' before, questa viene
! eseguita prima della visualizzazione del dialogo; inoltre se la proprieta'
! before restituisce true, il dialogo non viene mostrato (ovvero, le proprieta'
! description e after non vengono eseguite).
! Dopo che e' stata scelta un'opzione di dialogo, questa viene rimossa dalla
! lista a meno che non abbia settato l'attributo static, e tutti i suoi
! figli vengono invece aggiunti alla lista.
! Infine se l'opzione di dialogo ha l'attributo locked settato, l'opzione non
! viene usata, ne' viene mostrata nella statusline o nella finestra dei menu'
! (come se non fosse figlia del conversor)
!
! L'autore puo' far iniziare una conversazione con un certo png chiamando 
! la routine talkwith(png, flag), dove png e' il personaggio con cui
! iniziare una nuova conversazione e flag e' una variabile true/false che 
! indica se la conversazione deve essere iniziata anche se il protagonista 
! sta gia' parlando con qualcun'altro. Inoltre stoptalk() interrompe la
! conversazione (da usare anche alla fine del gioco).
! Potete usare la funzione talkcheck(png) per verificare se un certo
! png ha ancora opzioni di dialogo valide. La funzione talking() invece
! verifica se c'e' una conversazione attiva e, se e' cosi', restituisce
! il png con cui la conversazione e' in corso (altrimenti restituisce 0).
!
! Esempio
!
! NPC giorgio "Giorgio Parlanti"
!   with name 'giorgio' 'parlanti',
!   conversor giorgiotalk,
!   has male proper;
!
! Object giorgiotalk;
!
! Dialog -> gthello "Salve, Sig. Parlanti, come va?"
!   with description "~Salve a lei, signore.~";
!
! Dialog -> gtwhere "Dove sta andando di bello?"
!   with description [;
!      move gtbank to giorgiotalk;
!      "~Sto andando in banca.~";
!   ];
!
! Dialog gtbank "Ma oggi @`e sabato, la banca @`e chiusa."
!   with description "~E' vero. Grazie mille.~",
!   has static;
!
!
! Note:
! In Glulx la finestra dei dialoghi appare e scompare dinamicamente, secondo
! il bisogno. Se volete una finestra statica, prima di include wtalk_it.h 
! definite la costante STATIC_TALKWINDOW.
! Potete comunque usare i comandi TalkWindowOpen, TalkWindowReopen (quest'
! ultima riscrive le linee di dialogo) e TalkWindowClose per gestire
! la finestra dei dialoghi.
! Potete definire la costante TALK_WIN_UFRAME per avere una linea disegnata tra
! la finestra principale e quella di conversazione (da usare assieme alle 
! configurazioni senza frame di WinGlulxe).
!
! Questa estensione usa la routine entry point TimePasses() (che non fa altro
! che chiamare la routine dialogline.operate()) in modo da reagire in maniera
! corretta a demoni che spostano i png. Se avete bisogno di definire la vostra
! TimePasses(), inserite in quest'ultima la chiamata a dialogline.operate() e
! definite la costante TALK_NOTIMEPASSES prima di includere wtalk_it.h,
! oppure rinominate la vostra TimePasses() in CustomTimePasses().
!
! Se avete personalizzato la vostra statusline, rinominate la vostra procedura
! DrawStatusLine() in CustomDrawStatusLine() e togliete la direttiva
! Replace DrawStatusLine; in questo modo la vostra CustomDrawStatusLine()
! verra' chiamata opportunamente dalla DrawStatusLine di wtalk.
!
! ATTENZIONE:
! In Glulx, per fare in modo che la finestra dei dialoghi reagisca in modo
! corretto ai comandi SALVA, CARICA e RICOMINCIA, dovete aggiungere questo
!    TalkIdentifyGlkObject(phase,type,ref,rock);
! alla fine della vostra routine IdentifyGlkObject. Assicuratevi che tale
! riga venga eseguita in ogni chiamata di IdentifyGlkObject.
! Potete definire la costante CANT_TALK_ANSWER come una stringa o come il nome
! di una routine per personalizzare cosa viene stampato quando il giocatore
! prova a parlare a qualcuno ma non ci sono piu' argomenti disponibili.
! Potete anche definirla come NULL se non volete alcuna risposta.
! Se definite la costante WTALK_REMOVE_YESNO anche le azioni Yes e No verranno
! bloccate (come Tell, Ask, Answer, AskFor e tutti gli ordini)
! ------------------------------------------------------------------------------

!Global talkwith;

#Ifndef LIBRARY_PARSER;
!First inclusion
message "Using wtalk_it 1.4.9 by Paolo Lucchesi";
#IfDef TARGET_ZCODE;
Replace DrawStatusLine;
#Endif;
Replace TellSub;
Replace AskSub;
Replace AnswerSub;
Replace AskForSub;
#IfNDef WTALK_REMOVE_YESNO;
Replace YesSub;
Replace NoSub;
#EndIf;

Property conversor;

#IfDef TARGET_GLULX;
  Constant GG_TALKWIN_ROCK 297;
  Global gg_talkwin = 0;
#IfDef TALK_WIN_UFRAME;
  Constant GG_TALKWINUF_ROCK 296;
  Global gg_talkwinuf = 0;
#Endif;  
#Endif;

#Endif;

!-------------------------

#Ifdef LIBRARY_PARSER;
#Ifndef LIBRARY_GRAMMAR;
!Second inclusion

Global talkinited = false;

Object "(wtalk)" LibraryExtensions
  with ext_initialise [;
    !StartDaemon(dialogline);
    talkinited = true;
    TalkWindowClose();
  #Ifdef STATIC_TALKWINDOW;
    TalkWindowOpen();
  #Endif;
    rtrue;
];

Stub TalkInit 0; ! Legacy

#IfDef TARGET_GLULX;

[ TalkWindowOpen bg fg;
    if (gg_talkwin == 0) {
        gg_talkwin = glk_window_open(gg_mainwin, (winmethod_Below+winmethod_Fixed), 4, wintype_Textgrid, GG_TALKWIN_ROCK);
    }
    bg = 0; fg = $FFFFFF;
#IfDef TALK_WIN_UFRAME;
    if (glk_style_measure(gg_talkwin, style_Normal, stylehint_TextColor, gg_arguments)) 
        fg = gg_arguments --> 0;
    if (glk_style_measure(gg_talkwin, style_Normal, stylehint_BackColor, gg_arguments)) 
        bg = gg_arguments --> 0;
    if (gg_talkwinuf == 0) {
        gg_talkwinuf = glk_window_open(gg_mainwin, (winmethod_Below+winmethod_Fixed), 4, wintype_Graphics, GG_TALKWINUF_ROCK);
    }    
    glk_window_set_background_color(gg_talkwinuf, bg); 
    glk_window_clear(gg_talkwinuf);        
    glk_window_fill_rect(gg_talkwinuf,fg,1,1,2048,2);
#Endif;
    rtrue;
];

[ TalkWindowReopen;
    if (talkinited) dialogline.display();
];

[ TalkWindowClose;
    if (gg_talkwin) glk_window_close(gg_talkwin, 0);
    gg_talkwin = 0;
#IfDef TALK_WIN_UFRAME;
    if (gg_talkwinuf) glk_window_close(gg_talkwinuf, 0);
    gg_talkwinuf = 0;
#Endif;
    rtrue;
];

#Endif;
    
Stub TalkWindowOpen 0;
Stub TalkWindowReopen 0;
Stub TalkWindowClose 0;

#Ifndef TALK_NOTIMEPASSES;
[ TimePasses;
    dialogline.operate();
#Ifdef CustomTimePasses;
    CustomTimePasses();
#Endif;	
];
#Endif;

Object dialogline
  with ndialogs 0, dialog1 0, dialog2 0, dialog3 0,
    talker 0, lastturn false,
    operate [x y;
      self.ndialogs=0; self.dialog1=0; self.dialog2=0; self.dialog3=0;
      if (self.talker) if (~~TestScope(self.talker)) self.talker=0;
      !if (self.talker) if (self.talker notin real_location) { self.talker=0; return; }
      if (self.talker) {
          y=self.talker;
          objectloop (x in y.conversor) {
              if (x ofclass Dialog && x hasnt locked && self.ndialogs<3) {
                  switch(++self.ndialogs) {
                      1: self.dialog1 = x;
                      2: self.dialog2 = x;
                      3: self.dialog3 = x;
                  }
              }
          }
          if (self.ndialogs==0) self.talker=0;
      }
      self.display();
    ],
    display [lines tp i;          
      lines = self.ndialogs; if (lines > 3) lines = 3;
      tp = i; ! Just to avoid warnings
#Ifdef TARGET_GLULX;
    #Ifdef STATIC_TALKWINDOW;
      TalkWindowOpen();
    #Endif;
      if (self.ndialogs > 0) {
          TalkWindowOpen();
          if (gg_talkwin) {
              glk_set_window(gg_talkwin);
              glk_window_clear(gg_talkwin);
              glk_window_move_cursor(gg_talkwin,0,0);
              print "(Parlando con ", (the) self.talker, ")^";
              for (i=1: i<=lines: i++) {
                  switch(i) { 1: tp=self.dialog1;
                              2: tp=self.dialog2;
                              3: tp=self.dialog3;
                  }
                  glk_window_move_cursor(gg_talkwin,0,i);                  
                  print " ", i, ") ", (name) tp, "^";
              }
              MainWindow(); ! set_window
          }
      }
      else {
        #Ifdef STATIC_TALKWINDOW;
          if (gg_talkwin) {
              glk_set_window(gg_talkwin);
              glk_window_clear(gg_talkwin);
              MainWindow();
          }
        #Ifnot;
          TalkWindowClose();
        #Endif;
      }
#Endif;
    ];

[ talking;
    if (dialogline.ndialogs > 0) return dialogline.talker;
    rfalse;
];

#Ifdef TARGET_ZCODE;
Array dlstring buffer 80;

[ DrawStatusLine width posa posb lines i k tp;
![ DrawStatusLine width posa posb;
    posa = posb; ! warning squasher

    #Ifdef TARGET_GLULX;
    ! If we have no status window, we must not try to redraw it.
    if (gg_statuswin == 0)
        return;
    #Endif;

    ! If there is no player location, we shouldn't try to draw status window
    if (location == nothing || parent(player) == nothing)
        return;

    width = ScreenWidth();
#Ifdef TARGET_ZCODE;
    MoveCursor(1, 1);
    if (dialogline.ndialogs <= 0) {
        if (dialogline.lastturn) {
            StatusLineHeight(gg_statuswin_size+4);
            style roman;
            for (i=1: i<=4: i++) {MoveCursor(gg_statuswin_size+i, 1); spaces width; }
            style reverse;
        }
        StatusLineHeight(gg_statuswin_size);
    }
    else StatusLineHeight(gg_statuswin_size+4);
#Ifnot; ! TARGET_GLULX
    StatusLineHeight(gg_statuswin_size);
#Endif;

    MoveCursor(1, 1);
   
#Ifdef CustomDrawStatusLine;
    CustomDrawStatusLine();
#Ifnot;

    posa = width-26; posb = width-13;
    spaces width;

    MoveCursor(1, 2);
    if (location == thedark) {
        print (name) location;
    }
    else {
        FindVisibilityLevels();
        if (visibility_ceiling == location)
            print (name) location;
        else
            print (The) visibility_ceiling;
    }

#Ifdef TARGET_ZCODE;
  if (dialogline.ndialogs <= 0) {
#Endif;
    if (sys_statusline_flag && width > 53) {
        MoveCursor(1, posa);
        print (string) TIME__TX;
        LanguageTimeOfDay(sline1, sline2);
    }
    else {
        if (width > 66) {
            #Ifndef NO_SCORE;
            MoveCursor(1, posa);
            print (string) SCORE__TX, sline1;
            #Endif;
            MoveCursor(1, posb);
            print (string) MOVES__TX, sline2;
        }
        #Ifndef NO_SCORE;
        if (width > 53 && width <= 66) {
            MoveCursor(1, posb);
            print sline1, "/", sline2;
        }
        #Endif;
    }
#Ifdef TARGET_ZCODE;
  }
#Endif;

#Endif;

#Ifdef TARGET_ZCODE;
    dialogline.lastturn=false;
    if (dialogline.ndialogs > 0) {
       dialogline.lastturn=true;
       dlstring->0 = 78;
       @output_stream 3 dlstring;
       print "(Parlando con ", (the) dialogline.talker, ")";
       @output_stream -3;
       k = width - dlstring-->0 - 2;
       MoveCursor(1,k);
       print "(Parlando con ", (the) dialogline.talker, ")";

       for (i=2: i<=5: i++) { MoveCursor(i, 1); spaces width; }
       lines = dialogline.ndialogs; if (lines > 3) lines = 3;
       for (i=1: i<=lines: i++) {
           switch(i) { 1: tp=dialogline.dialog1;
                       2: tp=dialogline.dialog2;
                       3: tp=dialogline.dialog3;
                     }
           k=i+gg_statuswin_size; MoveCursor(k, 1); print "  ", i, ") ", (name) tp;
       }
    }
#Endif;

    MainWindow(); ! set_window
];

#Endif;

[ xinternalcanttalk;
  #Ifdef NPerson;
    if (NPerson==FIRST_PERSON) "Non ho niente da dire.";
  #Endif;
    "Non hai niente da dire.";
];

Default CANT_TALK_ANSWER xinternalcanttalk;

Class NPC
  with before [x;
    TalkTo:
      if (~~(self provides conversor)) rfalse;
      !r = true;
      !for (x = (child(self.conversor)) : x && r : x = sibling(x) ) if (x hasnt locked) r = false;
      if ((~~talkcheck(self)) && CANT_TALK_ANSWER ~= 0 or NULL) { PrintOrRunVar(CANT_TALK_ANSWER); rtrue; }   
      dialogline.talker=self;
      x=self.conversor;
      if (x provides before) x.before();
      dialogline.operate();
      rtrue;
  ],
  orders [; talktomsg(); rtrue; ],
has animate;

[ talkcheck obj x;
  if (~~(obj provides conversor)) rfalse;
  for (x = (child(obj.conversor)) : x : x = sibling(x) ) if (x hasnt locked) rtrue;
  rfalse;
];  

[ talkwith obj force x;
    if (dialogline.talker && ~~force) rtrue;
    if (obj == 0 or NULL) {
        dialogline.talker = 0;
        rtrue;
    }
    if (obj ofclass NPC && obj provides conversor) {
        dialogline.talker=obj;
        x=obj.conversor;
        if (x provides before) x.before();
        dialogline.operate();
        rtrue;
    }
    rfalse;
];

[ stoptalk;
    dialogline.talker = 0;
    dialogline.operate();
];

[ talktomsg; "Usa soltanto ~Parla con personaggio~."; ];

Class Dialog;

!--- per evitare warning
Object xwtalkxdummyx
   with xproc [; talkwith(); stoptalk(); TalkWindowOpen(); TalkWindowReopen();
                 TalkWindowClose(); TalkInit(); ];
   
[ TalkToSub;
  #Ifdef NPerson;
    if (noun == player && NPerson==FIRST_PERSON) "Better not to talk to myself.";
  #Endif;
    if (noun == player) "Meglio non parlare da soli.";
    print (The) noun;
    if (noun ofclass NPC) {
        if (noun has pluralname) print " non sembrano"; else print " non sembra";
        " voler rispondere.";
    }
    if (noun has pluralname) print " non sembrano capaci"; else print " non sembra capace";
    " di parlare.";
];

[ Dialog1Sub; DoDialog(1); ];
[ Dialog2Sub; DoDialog(2); ];
[ Dialog3Sub; DoDialog(3); ];

[ DoDialog op tp rval x y;
    if (dialogline.talker) if (~~TestScope(dialogline.talker)) dialogline.talker=0;
  #Ifdef NPerson;
    if (dialogline.talker==0 && NPerson==FIRST_PERSON) "Non sto parlando con nessuno, al momento...";
  #Endif;
    if (dialogline.talker==0) "Non stai parlando con nessuno, al momento.";
    if (op > dialogline.ndialogs) rfalse;
    switch(op) { 1: tp=dialogline.dialog1;
                 2: tp=dialogline.dialog2;
                 3: tp=dialogline.dialog3;
               }
    rval = false;
    if (tp provides before) if (tp.before()) rtrue;
    print "~", (name) tp, "~^";
    if (tp provides description) rval=tp.description();
    if (tp provides after) rval=tp.after();
    x = child(tp);
    while (x) {
        y = sibling(x);
        if (x ofclass Dialog) move x to parent(tp);
        x = y;
    }
    if (tp hasnt static) remove tp;
    return rval;
];

[ TellSub; talktomsg(); ];
[ AskSub; talktomsg(); ];
[ AnswerSub; talktomsg(); ];
[ AskForSub; talktomsg(); ];
#IfNDef WTALK_REMOVE_YESNO;
[ YesSub; talktomsg(); ];
[ NoSub; talktomsg(); ];
#EndIf;

#Ifdef TARGET_GLULX;
[ TalkIdentifyGlkObject phase type ref rock;
    !print "#", phase, "#"; KeyCharPrimitive();
    if (phase == 0) gg_talkwin = 0;
    if (phase == 1 && type == 0 && rock == GG_TALKWIN_ROCK) gg_talkwin = ref;
#IfDef TALK_WIN_UFRAME;
    if (phase == 0) gg_talkwinuf = 0;
    if (phase == 1 && type == 0 && rock == GG_TALKWINUF_ROCK) gg_talkwinuf = ref;
#Endif;
    if (phase == 2) {
        TalkWindowClose();
        dialogline.display();
    }
    rtrue;
];
#Endif;

#Endif;
#Endif;

!-------------------------------------

#Ifdef LIBRARY_GRAMMAR;
! Third inclusion
Extend 'parla' replace
                * 'a'/'ad'/'all^'/'allo'/'alla'/'al'/'agli'/'ai'/
                  'alle' creature 'circa'/'su'/'sul'/
                  'sullo'/'sull^'/'sulla'/'sugli'/'sui'/
                  'sulle'/'di'/'dello'/'della'/'dell^'/'dei'/
                  'degli'/'delle' topic
                                                 -> Tell
                * 'con'/'col' creature 'circa'/'su'/'sul'/
                  'sullo'/'sull^'/'sulla'/'sugli'/'sui'/
                  'sulle'/'di'/'dello'/'della'/'dell^'/'dei'/
                  'degli'/'delle' topic
                                                 -> Tell
                * 'a'/'ad'/'all^'/'allo'/'alla'/'al'/'agli'/'ai'/
                  'alle'/'con'/'col' creature
                                                 -> TalkTo;

Verb '1'        *                               -> Dialog1;
Verb '2'        *                               -> Dialog2;
Verb '3'        *                               -> Dialog3;
#Endif;
